@@ -1,3 +1,5 @@
+"""Message responsible for deleting other messages."""
+
 from typing import Any, Literal
 
 from langchain_core.messages.base import BaseMessage
@@ -9,7 +11,11 @@ class RemoveMessage(BaseMessage):
     type: Literal["remove"] = "remove"
     """The type of the message (used for serialization). Defaults to "remove"."""
 
-    def __init__(self, id: str, **kwargs: Any) -> None:
+    def __init__(
+        self,
+        id: str,
+        **kwargs: Any,
+    ) -> None:
         """Create a RemoveMessage.
 
         Args:
@@ -18,19 +24,10 @@ class RemoveMessage(BaseMessage):
 
         Raises:
             ValueError: If the 'content' field is passed in kwargs.
+
         """
         if kwargs.pop("content", None):
             msg = "RemoveMessage does not support 'content' field."
             raise ValueError(msg)
 
-        return super().__init__("", id=id, **kwargs)
-
-    @classmethod
-    def get_lc_namespace(cls) -> list[str]:
-        """Get the namespace of the langchain object.
-        Default is ["langchain", "schema", "messages"].
-        """
-        return ["langchain", "schema", "messages"]
-
-
-RemoveMessage.model_rebuild()
+        super().__init__("", id=id, **kwargs)
