@@ -1,3 +1,5 @@
+"""Configuration utilities for Runnables."""
+
 from __future__ import annotations
 
 import asyncio
@@ -6,25 +8,26 @@ import warnings
 from collections.abc import Awaitable, Generator, Iterable, Iterator, Sequence
 from concurrent.futures import Executor, Future, ThreadPoolExecutor
 from contextlib import contextmanager
-from contextvars import ContextVar, copy_context
+from contextvars import Context, ContextVar, Token, copy_context
 from functools import partial
 from typing import TYPE_CHECKING, Any, Callable, Optional, TypeVar, Union, cast
 
+from langsmith.run_helpers import _set_tracing_context, get_tracing_context
 from typing_extensions import ParamSpec, TypedDict
 
+from langchain_core.callbacks.manager import AsyncCallbackManager, CallbackManager
 from langchain_core.runnables.utils import (
     Input,
     Output,
     accepts_config,
     accepts_run_manager,
 )
+from langchain_core.tracers.langchain import LangChainTracer
 
 if TYPE_CHECKING:
     from langchain_core.callbacks.base import BaseCallbackManager, Callbacks
     from langchain_core.callbacks.manager import (
-        AsyncCallbackManager,
         AsyncCallbackManagerForChainRun,
-        CallbackManager,
         CallbackManagerForChainRun,
     )
 else:
@@ -115,15 +118,20 @@ var_child_runnable_config: ContextVar[RunnableConfig | None] = ContextVar(
 )
 
 
-def _set_config_context(config: RunnableConfig) -> None:
+# This is imported and used in langgraph, so don't break.
+def _set_config_context(
+    config: RunnableConfig,
+) -> tuple[Token[Optional[RunnableConfig]], Optional[dict[str, Any]]]:
     """Set the child Runnable config + tracing context.
 
     Args:
         config (RunnableConfig): The config to set.
-    """
-    from langchain_core.tracers.langchain import LangChainTracer
 
-    var_child_runnable_config.set(config)
+    Returns:
+        The token to reset the config and the previous tracing context.
+    """
+    config_token = var_child_runnable_config.set(config)
+    current_context = None
     if (
         (callbacks := config.get("callbacks"))
         and (
@@ -141,9 +149,38 @@ def _set_config_context(config: RunnableConfig) -> None:
         )
         and (run := tracer.run_map.get(str(parent_run_id)))
     ):
-        from langsmith.run_helpers import _set_tracing_context
-
+        current_context = get_tracing_context()
         _set_tracing_context({"parent": run})
+    return config_token, current_context
+
+
+@contextmanager
+def set_config_context(config: RunnableConfig) -> Generator[Context, None, None]:
+    """Set the child Runnable config + tracing context.
+
+    Args:
+        config (RunnableConfig): The config to set.
+
+    Yields:
+        The config context.
+    """
+    ctx = copy_context()
+    config_token, _ = ctx.run(_set_config_context, config)
+    try:
+        yield ctx
+    finally:
+        ctx.run(var_child_runnable_config.reset, config_token)
+        ctx.run(
+            _set_tracing_context,
+            {
+                "parent": None,
+                "project_name": None,
+                "tags": None,
+                "metadata": None,
+                "enabled": None,
+                "client": None,
+            },
+        )
 
 
 def ensure_config(config: Optional[RunnableConfig] = None) -> RunnableConfig:
@@ -166,7 +203,7 @@ def ensure_config(config: Optional[RunnableConfig] = None) -> RunnableConfig:
     if var_config := var_child_runnable_config.get():
         empty.update(
             cast(
-                RunnableConfig,
+                "RunnableConfig",
                 {
                     k: v.copy() if k in COPIABLE_KEYS else v  # type: ignore[attr-defined]
                     for k, v in var_config.items()
@@ -177,7 +214,7 @@ def ensure_config(config: Optional[RunnableConfig] = None) -> RunnableConfig:
     if config is not None:
         empty.update(
             cast(
-                RunnableConfig,
+                "RunnableConfig",
                 {
                     k: v.copy() if k in COPIABLE_KEYS else v  # type: ignore[attr-defined]
                     for k, v in config.items()
@@ -194,6 +231,7 @@ def ensure_config(config: Optional[RunnableConfig] = None) -> RunnableConfig:
             not key.startswith("__")
             and isinstance(value, (str, int, float, bool))
             and key not in empty["metadata"]
+            and key != "api_key"
         ):
             empty["metadata"][key] = value
     return empty
@@ -207,12 +245,12 @@ def get_config_list(
      It is useful for subclasses overriding batch() or abatch().
 
     Args:
-        config (Optional[Union[RunnableConfig, List[RunnableConfig]]]):
+        config (Optional[Union[RunnableConfig, list[RunnableConfig]]]):
           The config or list of configs.
         length (int): The length of the list.
 
     Returns:
-        List[RunnableConfig]: The list of configs.
+        list[RunnableConfig]: The list of configs.
 
     Raises:
         ValueError: If the length of the list is not equal to the length of the inputs.
@@ -237,7 +275,7 @@ def get_config_list(
             stacklevel=3,
         )
         subsequent = cast(
-            RunnableConfig, {k: v for k, v in config.items() if k != "run_id"}
+            "RunnableConfig", {k: v for k, v in config.items() if k != "run_id"}
         )
         return [
             ensure_config(subsequent) if i else ensure_config(config)
@@ -266,7 +304,7 @@ def patch_config(
         max_concurrency (Optional[int], optional): The max concurrency to set.
           Defaults to None.
         run_name (Optional[str], optional): The run name to set. Defaults to None.
-        configurable (Optional[Dict[str, Any]], optional): The configurable to set.
+        configurable (Optional[dict[str, Any]], optional): The configurable to set.
           Defaults to None.
 
     Returns:
@@ -307,18 +345,18 @@ def merge_configs(*configs: Optional[RunnableConfig]) -> RunnableConfig:
     for config in (ensure_config(c) for c in configs if c is not None):
         for key in config:
             if key == "metadata":
-                base[key] = {  # type: ignore
-                    **base.get(key, {}),  # type: ignore
-                    **(config.get(key) or {}),  # type: ignore
+                base["metadata"] = {
+                    **base.get("metadata", {}),
+                    **(config.get("metadata") or {}),
                 }
             elif key == "tags":
-                base[key] = sorted(  # type: ignore
-                    set(base.get(key, []) + (config.get(key) or [])),  # type: ignore
+                base["tags"] = sorted(
+                    set(base.get("tags", []) + (config.get("tags") or [])),
                 )
             elif key == "configurable":
-                base[key] = {  # type: ignore
-                    **base.get(key, {}),  # type: ignore
-                    **(config.get(key) or {}),  # type: ignore
+                base["configurable"] = {
+                    **base.get("configurable", {}),
+                    **(config.get("configurable") or {}),
                 }
             elif key == "callbacks":
                 base_callbacks = base.get("callbacks")
@@ -354,7 +392,7 @@ def merge_configs(*configs: Optional[RunnableConfig]) -> RunnableConfig:
             elif key in COPIABLE_KEYS and config[key] is not None:  # type: ignore[literal-required]
                 base[key] = config[key].copy()  # type: ignore[literal-required]
             else:
-                base[key] = config[key] or base.get(key)  # type: ignore
+                base[key] = config[key] or base.get(key)  # type: ignore[literal-required]
     return base
 
 
@@ -373,18 +411,14 @@ def call_func_with_variable_args(
     """Call function that may optionally accept a run_manager and/or config.
 
     Args:
-        func (Union[Callable[[Input], Output],
-          Callable[[Input, CallbackManagerForChainRun], Output],
-          Callable[[Input, CallbackManagerForChainRun, RunnableConfig], Output]]):
-           The function to call.
-        input (Input): The input to the function.
-        config (RunnableConfig): The config to pass to the function.
-        run_manager (CallbackManagerForChainRun): The run manager to
-          pass to the function. Defaults to None.
-        **kwargs (Any): The keyword arguments to pass to the function.
+        func: The function to call.
+        input: The input to the function.
+        config: The config to pass to the function.
+        run_manager: The run manager to pass to the function. Defaults to None.
+        **kwargs: The keyword arguments to pass to the function.
 
     Returns:
-        Output: The output of the function.
+        The output of the function.
     """
     if accepts_config(func):
         if run_manager is not None:
@@ -414,18 +448,14 @@ def acall_func_with_variable_args(
     """Async call function that may optionally accept a run_manager and/or config.
 
     Args:
-        func (Union[Callable[[Input], Awaitable[Output]], Callable[[Input,
-            AsyncCallbackManagerForChainRun], Awaitable[Output]], Callable[[Input,
-            AsyncCallbackManagerForChainRun, RunnableConfig], Awaitable[Output]]]):
-            The function to call.
-        input (Input): The input to the function.
-        config (RunnableConfig): The config to pass to the function.
-        run_manager (AsyncCallbackManagerForChainRun): The run manager
-          to pass to the function. Defaults to None.
-        **kwargs (Any): The keyword arguments to pass to the function.
+        func: The function to call.
+        input: The input to the function.
+        config: The config to pass to the function.
+        run_manager: The run manager to pass to the function. Defaults to None.
+        **kwargs: The keyword arguments to pass to the function.
 
     Returns:
-        Output: The output of the function.
+        The output of the function.
     """
     if accepts_config(func):
         if run_manager is not None:
@@ -446,8 +476,6 @@ def get_callback_manager_for_config(config: RunnableConfig) -> CallbackManager:
     Returns:
         CallbackManager: The callback manager.
     """
-    from langchain_core.callbacks.manager import CallbackManager
-
     return CallbackManager.configure(
         inheritable_callbacks=config.get("callbacks"),
         inheritable_tags=config.get("tags"),
@@ -466,8 +494,6 @@ def get_async_callback_manager_for_config(
     Returns:
         AsyncCallbackManager: The async callback manager.
     """
-    from langchain_core.callbacks.manager import AsyncCallbackManager
-
     return AsyncCallbackManager.configure(
         inheritable_callbacks=config.get("callbacks"),
         inheritable_tags=config.get("tags"),
@@ -499,7 +525,7 @@ class ContextThreadPoolExecutor(ThreadPoolExecutor):
             Future[T]: The future for the function.
         """
         return super().submit(
-            cast(Callable[..., T], partial(copy_context().run, func, *args, **kwargs))
+            cast("Callable[..., T]", partial(copy_context().run, func, *args, **kwargs))
         )
 
     def map(
@@ -569,9 +595,6 @@ async def run_in_executor(
 
     Returns:
         Output: The output of the function.
-
-    Raises:
-        RuntimeError: If the function raises a StopIteration.
     """
 
     def wrapper() -> T:
@@ -587,7 +610,7 @@ async def run_in_executor(
         # Use default executor with context copied from current context
         return await asyncio.get_running_loop().run_in_executor(
             None,
-            cast(Callable[..., T], partial(copy_context().run, wrapper)),
+            cast("Callable[..., T]", partial(copy_context().run, wrapper)),
         )
 
     return await asyncio.get_running_loop().run_in_executor(executor_or_config, wrapper)
