@@ -1,3 +1,5 @@
+"""Graph used in Runnables."""
+
 from __future__ import annotations
 
 import inspect
@@ -17,6 +19,8 @@ from typing import (
 )
 from uuid import UUID, uuid4
 
+from langchain_core.load.serializable import to_json_not_implemented
+from langchain_core.runnables.base import Runnable, RunnableSerializable
 from langchain_core.utils.pydantic import _IgnoreUnserializable, is_basemodel_subclass
 
 if TYPE_CHECKING:
@@ -28,7 +32,10 @@ if TYPE_CHECKING:
 
 
 class Stringifiable(Protocol):
-    def __str__(self) -> str: ...
+    """Protocol for objects that can be converted to a string."""
+
+    def __str__(self) -> str:
+        """Convert the object to a string."""
 
 
 class LabelsDict(TypedDict):
@@ -57,19 +64,16 @@ def is_uuid(value: str) -> bool:
 
 
 class Edge(NamedTuple):
-    """Edge in a graph.
-
-    Parameters:
-        source: The source node id.
-        target: The target node id.
-        data: Optional data associated with the edge. Defaults to None.
-        conditional: Whether the edge is conditional. Defaults to False.
-    """
+    """Edge in a graph."""
 
     source: str
+    """The source node id."""
     target: str
+    """The target node id."""
     data: Optional[Stringifiable] = None
+    """Optional data associated with the edge. Defaults to None."""
     conditional: bool = False
+    """Whether the edge is conditional. Defaults to False."""
 
     def copy(
         self, *, source: Optional[str] = None, target: Optional[str] = None
@@ -92,21 +96,23 @@ class Edge(NamedTuple):
 
 
 class Node(NamedTuple):
-    """Node in a graph.
-
-    Parameters:
-        id: The unique identifier of the node.
-        name: The name of the node.
-        data: The data of the node.
-        metadata: Optional metadata for the node. Defaults to None.
-    """
+    """Node in a graph."""
 
     id: str
+    """The unique identifier of the node."""
     name: str
-    data: Union[type[BaseModel], RunnableType]
+    """The name of the node."""
+    data: Union[type[BaseModel], RunnableType, None]
+    """The data of the node."""
     metadata: Optional[dict[str, Any]]
+    """Optional metadata for the node. Defaults to None."""
 
-    def copy(self, *, id: Optional[str] = None, name: Optional[str] = None) -> Node:
+    def copy(
+        self,
+        *,
+        id: Optional[str] = None,
+        name: Optional[str] = None,
+    ) -> Node:
         """Return a copy of the node with optional new id and name.
 
         Args:
@@ -125,16 +131,12 @@ class Node(NamedTuple):
 
 
 class Branch(NamedTuple):
-    """Branch in a graph.
-
-    Parameters:
-        condition: A callable that returns a string representation of the condition.
-        ends: Optional dictionary of end node ids for the branches. Defaults
-            to None.
-    """
+    """Branch in a graph."""
 
     condition: Callable[..., str]
+    """A callable that returns a string representation of the condition."""
     ends: Optional[dict[str, str]]
+    """Optional dictionary of end node ids for the branches. Defaults to None."""
 
 
 class CurveStyle(Enum):
@@ -158,7 +160,7 @@ class CurveStyle(Enum):
 class NodeStyles:
     """Schema for Hexadecimal color codes for different node types.
 
-    Parameters:
+    Args:
         default: The default color code. Defaults to "fill:#f2f0ff,line-height:1.2".
         first: The color code for the first node. Defaults to "fill-opacity:0".
         last: The color code for the last node. Defaults to "fill:#bfb6fc".
@@ -172,11 +174,16 @@ class NodeStyles:
 class MermaidDrawMethod(Enum):
     """Enum for different draw methods supported by Mermaid."""
 
-    PYPPETEER = "pyppeteer"  # Uses Pyppeteer to render the graph
-    API = "api"  # Uses Mermaid.INK API to render the graph
+    PYPPETEER = "pyppeteer"
+    """Uses Pyppeteer to render the graph"""
+    API = "api"
+    """Uses Mermaid.INK API to render the graph"""
 
 
-def node_data_str(id: str, data: Union[type[BaseModel], RunnableType]) -> str:
+def node_data_str(
+    id: str,
+    data: Union[type[BaseModel], RunnableType, None],
+) -> str:
     """Convert the data of a node to a string.
 
     Args:
@@ -186,14 +193,9 @@ def node_data_str(id: str, data: Union[type[BaseModel], RunnableType]) -> str:
     Returns:
         A string representation of the data.
     """
-    from langchain_core.runnables.base import Runnable
-
-    if not is_uuid(id):
+    if not is_uuid(id) or data is None:
         return id
-    elif isinstance(data, Runnable):
-        data_str = data.get_name()
-    else:
-        data_str = data.__name__
+    data_str = data.get_name() if isinstance(data, Runnable) else data.__name__
     return data_str if not data_str.startswith("Runnable") else data_str[8:]
 
 
@@ -210,11 +212,10 @@ def node_data_json(
     Returns:
         A dictionary with the type of the data and the data itself.
     """
-    from langchain_core.load.serializable import to_json_not_implemented
-    from langchain_core.runnables.base import Runnable, RunnableSerializable
-
-    if isinstance(node.data, RunnableSerializable):
-        json: dict[str, Any] = {
+    if node.data is None:
+        json: dict[str, Any] = {}
+    elif isinstance(node.data, RunnableSerializable):
+        json = {
             "type": "runnable",
             "data": {
                 "id": node.data.lc_id(),
@@ -257,7 +258,7 @@ def node_data_json(
 class Graph:
     """Graph of nodes and edges.
 
-    Parameters:
+    Args:
         nodes: Dictionary of nodes in the graph. Defaults to an empty dictionary.
         edges: List of edges in the graph. Defaults to an empty list.
     """
@@ -286,7 +287,7 @@ class Graph:
                 "target": stable_node_ids[edge.target],
             }
             if edge.data is not None:
-                edge_dict["data"] = edge.data
+                edge_dict["data"] = edge.data  # type: ignore[assignment]
             if edge.conditional:
                 edge_dict["conditional"] = True
             edges.append(edge_dict)
@@ -303,17 +304,19 @@ class Graph:
         }
 
     def __bool__(self) -> bool:
+        """Return whether the graph has any nodes."""
         return bool(self.nodes)
 
     def next_id(self) -> str:
-        """Return a new unique node
-        identifier that can be used to add a node to the graph.
+        """Return a new unique node identifier.
+
+        It that can be used to add a node to the graph.
         """
         return uuid4().hex
 
     def add_node(
         self,
-        data: Union[type[BaseModel], RunnableType],
+        data: Union[type[BaseModel], RunnableType, None],
         id: Optional[str] = None,
         *,
         metadata: Optional[dict[str, Any]] = None,
@@ -334,8 +337,8 @@ class Graph:
         if id is not None and id in self.nodes:
             msg = f"Node with id {id} already exists"
             raise ValueError(msg)
-        id = id or self.next_id()
-        node = Node(id=id, data=data, metadata=metadata, name=node_data_str(id, data))
+        id_ = id or self.next_id()
+        node = Node(id=id_, data=data, metadata=metadata, name=node_data_str(id_, data))
         self.nodes[node.id] = node
         return node
 
@@ -347,9 +350,7 @@ class Graph:
         """
         self.nodes.pop(node.id)
         self.edges = [
-            edge
-            for edge in self.edges
-            if edge.source != node.id and edge.target != node.id
+            edge for edge in self.edges if node.id not in {edge.source, edge.target}
         ]
 
     def add_edge(
@@ -357,7 +358,7 @@ class Graph:
         source: Node,
         target: Node,
         data: Optional[Stringifiable] = None,
-        conditional: bool = False,
+        conditional: bool = False,  # noqa: FBT001,FBT002
     ) -> Edge:
         """Add an edge to the graph and return it.
 
@@ -389,6 +390,7 @@ class Graph:
         self, graph: Graph, *, prefix: str = ""
     ) -> tuple[Optional[Node], Optional[Node]]:
         """Add all nodes and edges from another graph.
+
         Note this doesn't check for duplicates, nor does it connect the graphs.
 
         Args:
@@ -401,8 +403,8 @@ class Graph:
         if all(is_uuid(node.id) for node in graph.nodes.values()):
             prefix = ""
 
-        def prefixed(id: str) -> str:
-            return f"{prefix}:{id}" if prefix else id
+        def prefixed(id_: str) -> str:
+            return f"{prefix}:{id_}" if prefix else id_
 
         # prefix each node
         self.nodes.update(
@@ -423,8 +425,9 @@ class Graph:
         )
 
     def reid(self) -> Graph:
-        """Return a new graph with all nodes re-identified,
-        using their unique, readable names where possible.
+        """Return a new graph with all nodes re-identified.
+
+        Uses their unique, readable names where possible.
         """
         node_name_to_ids = defaultdict(list)
         for node in self.nodes.values():
@@ -440,13 +443,12 @@ class Graph:
             label = unique_labels[node_id]
             if is_uuid(node_id):
                 return label
-            else:
-                return node_id
+            return node_id
 
         return Graph(
             nodes={
-                _get_node_id(id): node.copy(id=_get_node_id(id))
-                for id, node in self.nodes.items()
+                _get_node_id(id_): node.copy(id=_get_node_id(id_))
+                for id_, node in self.nodes.items()
             },
             edges=[
                 edge.copy(
@@ -459,20 +461,31 @@ class Graph:
 
     def first_node(self) -> Optional[Node]:
         """Find the single node that is not a target of any edge.
+
         If there is no such node, or there are multiple, return None.
         When drawing the graph, this node would be the origin.
+
+        Returns:
+            The first node, or None if there is no such node or multiple
+            candidates.
         """
         return _first_node(self)
 
     def last_node(self) -> Optional[Node]:
         """Find the single node that is not a source of any edge.
+
         If there is no such node, or there are multiple, return None.
         When drawing the graph, this node would be the destination.
+
+        Returns:
+            The last node, or None if there is no such node or multiple
+            candidates.
         """
         return _last_node(self)
 
     def trim_first_node(self) -> None:
-        """Remove the first node if it exists and has a single outgoing edge,
+        """Remove the first node if it exists and has a single outgoing edge.
+
         i.e., if removing it would not leave the graph without a "first" node.
         """
         first_node = self.first_node()
@@ -484,7 +497,8 @@ class Graph:
             self.remove_node(first_node)
 
     def trim_last_node(self) -> None:
-        """Remove the last node if it exists and has a single incoming edge,
+        """Remove the last node if it exists and has a single incoming edge.
+
         i.e., if removing it would not leave the graph without a "last" node.
         """
         last_node = self.last_node()
@@ -496,8 +510,13 @@ class Graph:
             self.remove_node(last_node)
 
     def draw_ascii(self) -> str:
-        """Draw the graph as an ASCII art string."""
-        from langchain_core.runnables.graph_ascii import draw_ascii
+        """Draw the graph as an ASCII art string.
+
+        Returns:
+            The ASCII art string.
+        """
+        # Import locally to prevent circular import
+        from langchain_core.runnables.graph_ascii import draw_ascii  # noqa: PLC0415
 
         return draw_ascii(
             {node.id: node.name for node in self.nodes.values()},
@@ -541,7 +560,8 @@ class Graph:
         Returns:
             The PNG image as bytes if output_file_path is None, None otherwise.
         """
-        from langchain_core.runnables.graph_png import PngDrawer
+        # Import locally to prevent circular import
+        from langchain_core.runnables.graph_png import PngDrawer  # noqa: PLC0415
 
         default_node_labels = {node.id: node.name for node in self.nodes.values()}
 
@@ -563,6 +583,7 @@ class Graph:
         curve_style: CurveStyle = CurveStyle.LINEAR,
         node_colors: Optional[NodeStyles] = None,
         wrap_label_n_words: int = 9,
+        frontmatter_config: Optional[dict[str, Any]] = None,
     ) -> str:
         """Draw the graph as a Mermaid syntax string.
 
@@ -572,11 +593,30 @@ class Graph:
             node_colors: The colors of the nodes. Defaults to NodeStyles().
             wrap_label_n_words: The number of words to wrap the node labels at.
                 Defaults to 9.
+            frontmatter_config (dict[str, Any], optional): Mermaid frontmatter config.
+                Can be used to customize theme and styles. Will be converted to YAML and
+                added to the beginning of the mermaid graph. Defaults to None.
+
+                See more here: https://mermaid.js.org/config/configuration.html.
+
+                Example config:
+
+                .. code-block:: python
+
+                {
+                    "config": {
+                        "theme": "neutral",
+                        "look": "handDrawn",
+                        "themeVariables": { "primaryColor": "#e2e2e2"},
+                    }
+                }
+
 
         Returns:
             The Mermaid syntax string.
         """
-        from langchain_core.runnables.graph_mermaid import draw_mermaid
+        # Import locally to prevent circular import
+        from langchain_core.runnables.graph_mermaid import draw_mermaid  # noqa: PLC0415
 
         graph = self.reid()
         first_node = graph.first_node()
@@ -591,6 +631,7 @@ class Graph:
             curve_style=curve_style,
             node_styles=node_colors,
             wrap_label_n_words=wrap_label_n_words,
+            frontmatter_config=frontmatter_config,
         )
 
     def draw_mermaid_png(
@@ -603,6 +644,10 @@ class Graph:
         draw_method: MermaidDrawMethod = MermaidDrawMethod.API,
         background_color: str = "white",
         padding: int = 10,
+        max_retries: int = 1,
+        retry_delay: float = 1.0,
+        frontmatter_config: Optional[dict[str, Any]] = None,
+        base_url: Optional[str] = None,
     ) -> bytes:
         """Draw the graph as a PNG image using Mermaid.
 
@@ -617,16 +662,44 @@ class Graph:
                 Defaults to MermaidDrawMethod.API.
             background_color: The color of the background. Defaults to "white".
             padding: The padding around the graph. Defaults to 10.
+            max_retries: The maximum number of retries (MermaidDrawMethod.API).
+                Defaults to 1.
+            retry_delay: The delay between retries (MermaidDrawMethod.API).
+                Defaults to 1.0.
+            frontmatter_config (dict[str, Any], optional): Mermaid frontmatter config.
+                Can be used to customize theme and styles. Will be converted to YAML and
+                added to the beginning of the mermaid graph. Defaults to None.
+
+                See more here: https://mermaid.js.org/config/configuration.html.
+
+                Example config:
+
+                .. code-block:: python
+
+                {
+                    "config": {
+                        "theme": "neutral",
+                        "look": "handDrawn",
+                        "themeVariables": { "primaryColor": "#e2e2e2"},
+                    }
+                }
+            base_url: The base URL of the Mermaid server for rendering via API.
+                Defaults to None.
 
         Returns:
             The PNG image as bytes.
+
         """
-        from langchain_core.runnables.graph_mermaid import draw_mermaid_png
+        # Import locally to prevent circular import
+        from langchain_core.runnables.graph_mermaid import (  # noqa: PLC0415
+            draw_mermaid_png,
+        )
 
         mermaid_syntax = self.draw_mermaid(
             curve_style=curve_style,
             node_colors=node_colors,
             wrap_label_n_words=wrap_label_n_words,
+            frontmatter_config=frontmatter_config,
         )
         return draw_mermaid_png(
             mermaid_syntax=mermaid_syntax,
@@ -634,32 +707,39 @@ class Graph:
             draw_method=draw_method,
             background_color=background_color,
             padding=padding,
+            max_retries=max_retries,
+            retry_delay=retry_delay,
+            base_url=base_url,
         )
 
 
 def _first_node(graph: Graph, exclude: Sequence[str] = ()) -> Optional[Node]:
     """Find the single node that is not a target of any edge.
+
     Exclude nodes/sources with ids in the exclude list.
     If there is no such node, or there are multiple, return None.
     When drawing the graph, this node would be the origin.
     """
     targets = {edge.target for edge in graph.edges if edge.source not in exclude}
-    found: list[Node] = []
-    for node in graph.nodes.values():
-        if node.id not in exclude and node.id not in targets:
-            found.append(node)
+    found: list[Node] = [
+        node
+        for node in graph.nodes.values()
+        if node.id not in exclude and node.id not in targets
+    ]
     return found[0] if len(found) == 1 else None
 
 
 def _last_node(graph: Graph, exclude: Sequence[str] = ()) -> Optional[Node]:
     """Find the single node that is not a source of any edge.
+
     Exclude nodes/targets with ids in the exclude list.
     If there is no such node, or there are multiple, return None.
     When drawing the graph, this node would be the destination.
     """
     sources = {edge.source for edge in graph.edges if edge.target not in exclude}
-    found: list[Node] = []
-    for node in graph.nodes.values():
-        if node.id not in exclude and node.id not in sources:
-            found.append(node)
+    found: list[Node] = [
+        node
+        for node in graph.nodes.values()
+        if node.id not in exclude and node.id not in sources
+    ]
     return found[0] if len(found) == 1 else None
