@@ -1,8 +1,9 @@
 from collections.abc import AsyncIterator, Iterator
 from typing import Any
 
+import pydantic
 import pytest
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, Field, ValidationError
 
 from langchain_core.messages import (
     AIMessage,
@@ -16,7 +17,6 @@ from langchain_core.output_parsers.openai_tools import (
     PydanticToolsParser,
 )
 from langchain_core.outputs import ChatGeneration
-from langchain_core.utils.pydantic import PYDANTIC_MAJOR_VERSION
 
 STREAMED_MESSAGES: list = [
     AIMessageChunk(content=""),
@@ -362,7 +362,7 @@ EXPECTED_STREAMED_JSON = [
 ]
 
 
-def _get_iter(use_tool_calls: bool = False) -> Any:
+def _get_iter(*, use_tool_calls: bool = False) -> Any:
     if use_tool_calls:
         list_to_iter = STREAMED_MESSAGES_WITH_TOOL_CALLS
     else:
@@ -374,7 +374,7 @@ def _get_iter(use_tool_calls: bool = False) -> Any:
     return input_iter
 
 
-def _get_aiter(use_tool_calls: bool = False) -> Any:
+def _get_aiter(*, use_tool_calls: bool = False) -> Any:
     if use_tool_calls:
         list_to_iter = STREAMED_MESSAGES_WITH_TOOL_CALLS
     else:
@@ -387,92 +387,364 @@ def _get_aiter(use_tool_calls: bool = False) -> Any:
     return input_iter
 
 
-def test_partial_json_output_parser() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_iter(use_tool_calls)
-        chain = input_iter | JsonOutputToolsParser()
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_partial_json_output_parser(*, use_tool_calls: bool) -> None:
+    input_iter = _get_iter(use_tool_calls=use_tool_calls)
+    chain = input_iter | JsonOutputToolsParser()
 
-        actual = list(chain.stream(None))
-        expected: list = [[]] + [
-            [{"type": "NameCollector", "args": chunk}]
-            for chunk in EXPECTED_STREAMED_JSON
+    actual = list(chain.stream(None))
+    expected: list = [[]] + [
+        [{"type": "NameCollector", "args": chunk}] for chunk in EXPECTED_STREAMED_JSON
+    ]
+    assert actual == expected
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+async def test_partial_json_output_parser_async(*, use_tool_calls: bool) -> None:
+    input_iter = _get_aiter(use_tool_calls=use_tool_calls)
+    chain = input_iter | JsonOutputToolsParser()
+
+    actual = [p async for p in chain.astream(None)]
+    expected: list = [[]] + [
+        [{"type": "NameCollector", "args": chunk}] for chunk in EXPECTED_STREAMED_JSON
+    ]
+    assert actual == expected
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_partial_json_output_parser_return_id(*, use_tool_calls: bool) -> None:
+    input_iter = _get_iter(use_tool_calls=use_tool_calls)
+    chain = input_iter | JsonOutputToolsParser(return_id=True)
+
+    actual = list(chain.stream(None))
+    expected: list = [[]] + [
+        [
+            {
+                "type": "NameCollector",
+                "args": chunk,
+                "id": "call_OwL7f5PEPJTYzw9sQlNJtCZl",
+            }
         ]
-        assert actual == expected
+        for chunk in EXPECTED_STREAMED_JSON
+    ]
+    assert actual == expected
 
 
-async def test_partial_json_output_parser_async() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_aiter(use_tool_calls)
-        chain = input_iter | JsonOutputToolsParser()
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_partial_json_output_key_parser(*, use_tool_calls: bool) -> None:
+    input_iter = _get_iter(use_tool_calls=use_tool_calls)
+    chain = input_iter | JsonOutputKeyToolsParser(key_name="NameCollector")
 
-        actual = [p async for p in chain.astream(None)]
-        expected: list = [[]] + [
-            [{"type": "NameCollector", "args": chunk}]
-            for chunk in EXPECTED_STREAMED_JSON
+    actual = list(chain.stream(None))
+    expected: list = [[]] + [[chunk] for chunk in EXPECTED_STREAMED_JSON]
+    assert actual == expected
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+async def test_partial_json_output_parser_key_async(*, use_tool_calls: bool) -> None:
+    input_iter = _get_aiter(use_tool_calls=use_tool_calls)
+
+    chain = input_iter | JsonOutputKeyToolsParser(key_name="NameCollector")
+
+    actual = [p async for p in chain.astream(None)]
+    expected: list = [[]] + [[chunk] for chunk in EXPECTED_STREAMED_JSON]
+    assert actual == expected
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_partial_json_output_key_parser_first_only(*, use_tool_calls: bool) -> None:
+    input_iter = _get_iter(use_tool_calls=use_tool_calls)
+
+    chain = input_iter | JsonOutputKeyToolsParser(
+        key_name="NameCollector", first_tool_only=True
+    )
+
+    assert list(chain.stream(None)) == EXPECTED_STREAMED_JSON
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+async def test_partial_json_output_parser_key_async_first_only(
+    *,
+    use_tool_calls: bool,
+) -> None:
+    input_iter = _get_aiter(use_tool_calls=use_tool_calls)
+
+    chain = input_iter | JsonOutputKeyToolsParser(
+        key_name="NameCollector", first_tool_only=True
+    )
+
+    assert [p async for p in chain.astream(None)] == EXPECTED_STREAMED_JSON
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_json_output_key_tools_parser_multiple_tools_first_only(
+    *, use_tool_calls: bool
+) -> None:
+    # Test case from the original bug report
+    def create_message() -> AIMessage:
+        tool_calls_data = [
+            {
+                "id": "call_other",
+                "function": {"name": "other", "arguments": '{"b":2}'},
+                "type": "function",
+            },
+            {
+                "id": "call_func",
+                "function": {"name": "func", "arguments": '{"a":1}'},
+                "type": "function",
+            },
         ]
-        assert actual == expected
 
-
-def test_partial_json_output_parser_return_id() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_iter(use_tool_calls)
-        chain = input_iter | JsonOutputToolsParser(return_id=True)
-
-        actual = list(chain.stream(None))
-        expected: list = [[]] + [
-            [
-                {
-                    "type": "NameCollector",
-                    "args": chunk,
-                    "id": "call_OwL7f5PEPJTYzw9sQlNJtCZl",
-                }
-            ]
-            for chunk in EXPECTED_STREAMED_JSON
-        ]
-        assert actual == expected
-
-
-def test_partial_json_output_key_parser() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_iter(use_tool_calls)
-        chain = input_iter | JsonOutputKeyToolsParser(key_name="NameCollector")
-
-        actual = list(chain.stream(None))
-        expected: list = [[]] + [[chunk] for chunk in EXPECTED_STREAMED_JSON]
-        assert actual == expected
-
-
-async def test_partial_json_output_parser_key_async() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_aiter(use_tool_calls)
-
-        chain = input_iter | JsonOutputKeyToolsParser(key_name="NameCollector")
-
-        actual = [p async for p in chain.astream(None)]
-        expected: list = [[]] + [[chunk] for chunk in EXPECTED_STREAMED_JSON]
-        assert actual == expected
-
-
-def test_partial_json_output_key_parser_first_only() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_iter(use_tool_calls)
-
-        chain = input_iter | JsonOutputKeyToolsParser(
-            key_name="NameCollector", first_tool_only=True
+        if use_tool_calls:
+            return AIMessage(
+                content="",
+                tool_calls=[
+                    {"id": "call_other", "name": "other", "args": {"b": 2}},
+                    {"id": "call_func", "name": "func", "args": {"a": 1}},
+                ],
+            )
+        return AIMessage(
+            content="",
+            additional_kwargs={"tool_calls": tool_calls_data},
         )
 
-        assert list(chain.stream(None)) == EXPECTED_STREAMED_JSON
+    result = [ChatGeneration(message=create_message())]
+
+    # Test with return_id=True
+    parser = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=True
+    )
+    output = parser.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return the func tool call, not None
+    assert output is not None
+    assert output["type"] == "func"
+    assert output["args"] == {"a": 1}
+    assert "id" in output
+
+    # Test with return_id=False
+    parser_no_id = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=False
+    )
+    output_no_id = parser_no_id.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return just the args
+    assert output_no_id == {"a": 1}
 
 
-async def test_partial_json_output_parser_key_async_first_only() -> None:
-    for use_tool_calls in [False, True]:
-        input_iter = _get_aiter(use_tool_calls)
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_json_output_key_tools_parser_multiple_tools_no_match(
+    *, use_tool_calls: bool
+) -> None:
+    def create_message() -> AIMessage:
+        tool_calls_data = [
+            {
+                "id": "call_other",
+                "function": {"name": "other", "arguments": '{"b":2}'},
+                "type": "function",
+            },
+            {
+                "id": "call_another",
+                "function": {"name": "another", "arguments": '{"c":3}'},
+                "type": "function",
+            },
+        ]
 
-        chain = input_iter | JsonOutputKeyToolsParser(
-            key_name="NameCollector", first_tool_only=True
+        if use_tool_calls:
+            return AIMessage(
+                content="",
+                tool_calls=[
+                    {"id": "call_other", "name": "other", "args": {"b": 2}},
+                    {"id": "call_another", "name": "another", "args": {"c": 3}},
+                ],
+            )
+        return AIMessage(
+            content="",
+            additional_kwargs={"tool_calls": tool_calls_data},
         )
 
-        assert [p async for p in chain.astream(None)] == EXPECTED_STREAMED_JSON
+    result = [ChatGeneration(message=create_message())]
+
+    # Test with return_id=True, first_tool_only=True
+    parser = JsonOutputKeyToolsParser(
+        key_name="nonexistent", first_tool_only=True, return_id=True
+    )
+    output = parser.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return None when no matches
+    assert output is None
+
+    # Test with return_id=False, first_tool_only=True
+    parser_no_id = JsonOutputKeyToolsParser(
+        key_name="nonexistent", first_tool_only=True, return_id=False
+    )
+    output_no_id = parser_no_id.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return None when no matches
+    assert output_no_id is None
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_json_output_key_tools_parser_multiple_matching_tools(
+    *, use_tool_calls: bool
+) -> None:
+    def create_message() -> AIMessage:
+        tool_calls_data = [
+            {
+                "id": "call_func1",
+                "function": {"name": "func", "arguments": '{"a":1}'},
+                "type": "function",
+            },
+            {
+                "id": "call_other",
+                "function": {"name": "other", "arguments": '{"b":2}'},
+                "type": "function",
+            },
+            {
+                "id": "call_func2",
+                "function": {"name": "func", "arguments": '{"a":3}'},
+                "type": "function",
+            },
+        ]
+
+        if use_tool_calls:
+            return AIMessage(
+                content="",
+                tool_calls=[
+                    {"id": "call_func1", "name": "func", "args": {"a": 1}},
+                    {"id": "call_other", "name": "other", "args": {"b": 2}},
+                    {"id": "call_func2", "name": "func", "args": {"a": 3}},
+                ],
+            )
+        return AIMessage(
+            content="",
+            additional_kwargs={"tool_calls": tool_calls_data},
+        )
+
+    result = [ChatGeneration(message=create_message())]
+
+    # Test with first_tool_only=True - should return first matching
+    parser = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=True
+    )
+    output = parser.parse_result(result)  # type: ignore[arg-type]
+
+    assert output is not None
+    assert output["type"] == "func"
+    assert output["args"] == {"a": 1}  # First matching tool call
+
+    # Test with first_tool_only=False - should return all matching
+    parser_all = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=False, return_id=True
+    )
+    output_all = parser_all.parse_result(result)  # type: ignore[arg-type]
+
+    assert len(output_all) == 2
+    assert output_all[0]["args"] == {"a": 1}
+    assert output_all[1]["args"] == {"a": 3}
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_json_output_key_tools_parser_empty_results(*, use_tool_calls: bool) -> None:
+    def create_message() -> AIMessage:
+        if use_tool_calls:
+            return AIMessage(content="", tool_calls=[])
+        return AIMessage(content="", additional_kwargs={"tool_calls": []})
+
+    result = [ChatGeneration(message=create_message())]
+
+    # Test with first_tool_only=True
+    parser = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=True
+    )
+    output = parser.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return None for empty results
+    assert output is None
+
+    # Test with first_tool_only=False
+    parser_all = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=False, return_id=True
+    )
+    output_all = parser_all.parse_result(result)  # type: ignore[arg-type]
+
+    # Should return empty list for empty results
+    assert output_all == []
+
+
+@pytest.mark.parametrize("use_tool_calls", [False, True])
+def test_json_output_key_tools_parser_parameter_combinations(
+    *, use_tool_calls: bool
+) -> None:
+    """Test all parameter combinations of JsonOutputKeyToolsParser."""
+
+    def create_message() -> AIMessage:
+        tool_calls_data = [
+            {
+                "id": "call_other",
+                "function": {"name": "other", "arguments": '{"b":2}'},
+                "type": "function",
+            },
+            {
+                "id": "call_func1",
+                "function": {"name": "func", "arguments": '{"a":1}'},
+                "type": "function",
+            },
+            {
+                "id": "call_func2",
+                "function": {"name": "func", "arguments": '{"a":3}'},
+                "type": "function",
+            },
+        ]
+
+        if use_tool_calls:
+            return AIMessage(
+                content="",
+                tool_calls=[
+                    {"id": "call_other", "name": "other", "args": {"b": 2}},
+                    {"id": "call_func1", "name": "func", "args": {"a": 1}},
+                    {"id": "call_func2", "name": "func", "args": {"a": 3}},
+                ],
+            )
+        return AIMessage(
+            content="",
+            additional_kwargs={"tool_calls": tool_calls_data},
+        )
+
+    result: list[ChatGeneration] = [ChatGeneration(message=create_message())]
+
+    # Test: first_tool_only=True, return_id=True
+    parser1 = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=True
+    )
+    output1 = parser1.parse_result(result)  # type: ignore[arg-type]
+    assert output1["type"] == "func"
+    assert output1["args"] == {"a": 1}
+    assert "id" in output1
+
+    # Test: first_tool_only=True, return_id=False
+    parser2 = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=True, return_id=False
+    )
+    output2 = parser2.parse_result(result)  # type: ignore[arg-type]
+    assert output2 == {"a": 1}
+
+    # Test: first_tool_only=False, return_id=True
+    parser3 = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=False, return_id=True
+    )
+    output3 = parser3.parse_result(result)  # type: ignore[arg-type]
+    assert len(output3) == 2
+    assert all("id" in item for item in output3)
+    assert output3[0]["args"] == {"a": 1}
+    assert output3[1]["args"] == {"a": 3}
+
+    # Test: first_tool_only=False, return_id=False
+    parser4 = JsonOutputKeyToolsParser(
+        key_name="func", first_tool_only=False, return_id=False
+    )
+    output4 = parser4.parse_result(result)  # type: ignore[arg-type]
+    assert output4 == [{"a": 1}, {"a": 3}]
 
 
 class Person(BaseModel):
@@ -507,7 +779,7 @@ EXPECTED_STREAMED_PYDANTIC = [
 
 def test_partial_pydantic_output_parser() -> None:
     for use_tool_calls in [False, True]:
-        input_iter = _get_iter(use_tool_calls)
+        input_iter = _get_iter(use_tool_calls=use_tool_calls)
 
         chain = input_iter | PydanticToolsParser(
             tools=[NameCollector], first_tool_only=True
@@ -519,7 +791,7 @@ def test_partial_pydantic_output_parser() -> None:
 
 async def test_partial_pydantic_output_parser_async() -> None:
     for use_tool_calls in [False, True]:
-        input_iter = _get_aiter(use_tool_calls)
+        input_iter = _get_aiter(use_tool_calls=use_tool_calls)
 
         chain = input_iter | PydanticToolsParser(
             tools=[NameCollector], first_tool_only=True
@@ -529,10 +801,8 @@ async def test_partial_pydantic_output_parser_async() -> None:
         assert actual == EXPECTED_STREAMED_PYDANTIC
 
 
-@pytest.mark.skipif(PYDANTIC_MAJOR_VERSION != 2, reason="This test is for pydantic 2")
 def test_parse_with_different_pydantic_2_v1() -> None:
     """Test with pydantic.v1.BaseModel from pydantic 2."""
-    import pydantic
 
     class Forecast(pydantic.v1.BaseModel):
         temperature: int
@@ -540,7 +810,7 @@ def test_parse_with_different_pydantic_2_v1() -> None:
 
     # Can't get pydantic to work here due to the odd typing of tryig to support
     # both v1 and v2 in the same codebase.
-    parser = PydanticToolsParser(tools=[Forecast])  # type: ignore[list-item]
+    parser = PydanticToolsParser(tools=[Forecast])
     message = AIMessage(
         content="",
         tool_calls=[
@@ -564,18 +834,16 @@ def test_parse_with_different_pydantic_2_v1() -> None:
     ]
 
 
-@pytest.mark.skipif(PYDANTIC_MAJOR_VERSION != 2, reason="This test is for pydantic 2")
 def test_parse_with_different_pydantic_2_proper() -> None:
     """Test with pydantic.BaseModel from pydantic 2."""
-    import pydantic
 
-    class Forecast(pydantic.BaseModel):
+    class Forecast(BaseModel):
         temperature: int
         forecast: str
 
     # Can't get pydantic to work here due to the odd typing of tryig to support
     # both v1 and v2 in the same codebase.
-    parser = PydanticToolsParser(tools=[Forecast])  # type: ignore[list-item]
+    parser = PydanticToolsParser(tools=[Forecast])
     message = AIMessage(
         content="",
         tool_calls=[
@@ -599,36 +867,22 @@ def test_parse_with_different_pydantic_2_proper() -> None:
     ]
 
 
-@pytest.mark.skipif(PYDANTIC_MAJOR_VERSION != 1, reason="This test is for pydantic 1")
-def test_parse_with_different_pydantic_1_proper() -> None:
-    """Test with pydantic.BaseModel from pydantic 1."""
-    import pydantic
-
-    class Forecast(pydantic.BaseModel):
-        temperature: int
-        forecast: str
-
-    # Can't get pydantic to work here due to the odd typing of tryig to support
-    # both v1 and v2 in the same codebase.
-    parser = PydanticToolsParser(tools=[Forecast])  # type: ignore[list-item]
+def test_max_tokens_error(caplog: Any) -> None:
+    parser = PydanticToolsParser(tools=[NameCollector], first_tool_only=True)
     message = AIMessage(
         content="",
         tool_calls=[
             {
                 "id": "call_OwL7f5PE",
-                "name": "Forecast",
-                "args": {"temperature": 20, "forecast": "Sunny"},
+                "name": "NameCollector",
+                "args": {"names": ["suz", "jerm"]},
             }
         ],
+        response_metadata={"stop_reason": "max_tokens"},
     )
-
-    generation = ChatGeneration(
-        message=message,
+    with pytest.raises(ValidationError):
+        _ = parser.invoke(message)
+    assert any(
+        "`max_tokens` stop reason" in msg and record.levelname == "ERROR"
+        for record, msg in zip(caplog.records, caplog.messages)
     )
-
-    assert parser.parse_result([generation]) == [
-        Forecast(
-            temperature=20,
-            forecast="Sunny",
-        )
-    ]
