@@ -1,10 +1,12 @@
+import re
 import warnings
 from pathlib import Path
 from typing import Any, Union, cast
 
 import pytest
+from packaging import version
 from pydantic import ValidationError
-from syrupy import SnapshotAssertion
+from syrupy.assertion import SnapshotAssertion
 
 from langchain_core._api.deprecation import (
     LangChainPendingDeprecationWarning,
@@ -13,27 +15,32 @@ from langchain_core.load import dumpd, load
 from langchain_core.messages import (
     AIMessage,
     BaseMessage,
+    ChatMessage,
     HumanMessage,
     SystemMessage,
+    ToolMessage,
     get_buffer_string,
 )
 from langchain_core.prompt_values import ChatPromptValue
 from langchain_core.prompts import PromptTemplate
 from langchain_core.prompts.chat import (
     AIMessagePromptTemplate,
-    BaseMessagePromptTemplate,
-    ChatMessage,
     ChatMessagePromptTemplate,
     ChatPromptTemplate,
     HumanMessagePromptTemplate,
     MessagesPlaceholder,
     SystemMessagePromptTemplate,
-    _convert_to_message,
+    _convert_to_message_template,
 )
+from langchain_core.prompts.message import BaseMessagePromptTemplate
 from langchain_core.prompts.string import PromptTemplateFormat
-from langchain_core.utils.pydantic import PYDANTIC_MAJOR_VERSION, PYDANTIC_MINOR_VERSION
+from langchain_core.utils.pydantic import (
+    PYDANTIC_VERSION,
+)
 from tests.unit_tests.pydantic_utils import _normalize_schema
 
+CUR_DIR = Path(__file__).parent.absolute().resolve()
+
 
 @pytest.fixture
 def messages() -> list[BaseMessagePromptTemplate]:
@@ -78,7 +85,7 @@ def chat_prompt_template(
     """Create a chat prompt template."""
     return ChatPromptTemplate(
         input_variables=["foo", "bar", "context"],
-        messages=messages,  # type: ignore[arg-type]
+        messages=messages,
     )
 
 
@@ -165,15 +172,14 @@ def test_create_system_message_prompt_list_template_partial_variables_not_null()
         {variables}
         """
 
-    try:
-        graph_analyst_template = SystemMessagePromptTemplate.from_template(
+    with pytest.raises(
+        ValueError, match="Partial variables are not supported for list of templates"
+    ):
+        _ = SystemMessagePromptTemplate.from_template(
             template=[graph_creator_content1, graph_creator_content2],
             input_variables=["variables"],
             partial_variables={"variables": "foo"},
         )
-        graph_analyst_template.format(variables="foo")
-    except ValueError as e:
-        assert str(e) == "Partial variables are not supported for list of templates."
 
 
 def test_message_prompt_template_from_template_file() -> None:
@@ -330,7 +336,7 @@ def test_chat_prompt_template_from_messages_jinja2() -> None:
 
 @pytest.mark.requires("jinja2")
 @pytest.mark.parametrize(
-    "template_format,image_type_placeholder,image_data_placeholder",
+    ("template_format", "image_type_placeholder", "image_data_placeholder"),
     [
         ("f-string", "{image_type}", "{image_data}"),
         ("mustache", "{{image_type}}", "{{image_data}}"),
@@ -374,7 +380,10 @@ def test_chat_prompt_template_with_messages(
     messages: list[BaseMessagePromptTemplate],
 ) -> None:
     chat_prompt_template = ChatPromptTemplate.from_messages(
-        messages + [HumanMessage(content="foo")]
+        [
+            *messages,
+            HumanMessage(content="foo"),
+        ]
     )
     assert sorted(chat_prompt_template.input_variables) == sorted(
         [
@@ -393,35 +402,43 @@ def test_chat_prompt_template_with_messages(
 
 def test_chat_invalid_input_variables_extra() -> None:
     messages = [HumanMessage(content="foo")]
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Got mismatched input_variables. Expected: set(). Got: ['foo']"
+        ),
+    ):
         ChatPromptTemplate(
-            messages=messages,  # type: ignore[arg-type]
+            messages=messages,
             input_variables=["foo"],
-            validate_template=True,  # type: ignore[arg-type]
+            validate_template=True,
         )
     assert (
-        ChatPromptTemplate(messages=messages, input_variables=["foo"]).input_variables  # type: ignore[arg-type]
+        ChatPromptTemplate(messages=messages, input_variables=["foo"]).input_variables
         == []
     )
 
 
 def test_chat_invalid_input_variables_missing() -> None:
     messages = [HumanMessagePromptTemplate.from_template("{foo}")]
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape("Got mismatched input_variables. Expected: {'foo'}. Got: []"),
+    ):
         ChatPromptTemplate(
-            messages=messages,  # type: ignore[arg-type]
+            messages=messages,
             input_variables=[],
-            validate_template=True,  # type: ignore[arg-type]
+            validate_template=True,
         )
     assert ChatPromptTemplate(
-        messages=messages,  # type: ignore[arg-type]
-        input_variables=[],  # type: ignore[arg-type]
+        messages=messages,
+        input_variables=[],
     ).input_variables == ["foo"]
 
 
 def test_infer_variables() -> None:
     messages = [HumanMessagePromptTemplate.from_template("{foo}")]
-    prompt = ChatPromptTemplate(messages=messages)  # type: ignore[arg-type, call-arg]
+    prompt = ChatPromptTemplate(messages=messages)
     assert prompt.input_variables == ["foo"]
 
 
@@ -432,7 +449,7 @@ def test_chat_valid_with_partial_variables() -> None:
         )
     ]
     prompt = ChatPromptTemplate(
-        messages=messages,  # type: ignore[arg-type]
+        messages=messages,
         input_variables=["question", "context"],
         partial_variables={"formatins": "some structure"},
     )
@@ -446,9 +463,9 @@ def test_chat_valid_infer_variables() -> None:
             "Do something with {question} using {context} giving it like {formatins}"
         )
     ]
-    prompt = ChatPromptTemplate(  # type: ignore[call-arg]
-        messages=messages,  # type: ignore[arg-type]
-        partial_variables={"formatins": "some structure"},  # type: ignore[arg-type]
+    prompt = ChatPromptTemplate(
+        messages=messages,
+        partial_variables={"formatins": "some structure"},
     )
     assert set(prompt.input_variables) == {"question", "context"}
     assert prompt.partial_variables == {"formatins": "some structure"}
@@ -481,7 +498,7 @@ async def test_chat_from_role_strings() -> None:
 
 
 @pytest.mark.parametrize(
-    "args,expected",
+    ("args", "expected"),
     [
         (
             ("human", "{question}"),
@@ -510,7 +527,7 @@ def test_convert_to_message(
     args: Any, expected: Union[BaseMessage, BaseMessagePromptTemplate]
 ) -> None:
     """Test convert to message."""
-    assert _convert_to_message(args) == expected
+    assert _convert_to_message_template(args) == expected
 
 
 def test_chat_prompt_template_indexing() -> None:
@@ -551,11 +568,11 @@ def test_chat_prompt_template_append_and_extend() -> None:
 
 def test_convert_to_message_is_strict() -> None:
     """Verify that _convert_to_message is strict."""
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="Unexpected message type: meow"):
         # meow does not correspond to a valid message type.
         # this test is here to ensure that functionality to interpret `meow`
         # as a role is NOT added.
-        _convert_to_message(("meow", "question"))
+        _convert_to_message_template(("meow", "question"))
 
 
 def test_chat_message_partial() -> None:
@@ -582,6 +599,23 @@ def test_chat_message_partial() -> None:
     assert template2.format(input="hello") == get_buffer_string(expected)
 
 
+def test_chat_message_partial_composition() -> None:
+    """Test composition of partially initialized messages."""
+    prompt = ChatPromptTemplate.from_messages([("system", "Prompt {x} {y}")]).partial(
+        x="1"
+    )
+
+    appendix = ChatPromptTemplate.from_messages([("system", "Appendix {z}")])
+
+    res = (prompt + appendix).format_messages(y="2", z="3")
+    expected = [
+        SystemMessage(content="Prompt 1 2"),
+        SystemMessage(content="Appendix 3"),
+    ]
+
+    assert res == expected
+
+
 async def test_chat_tmpl_from_messages_multipart_text() -> None:
     template = ChatPromptTemplate.from_messages(
         [
@@ -724,7 +758,7 @@ async def test_chat_tmpl_from_messages_multipart_image() -> None:
 
 async def test_chat_tmpl_from_messages_multipart_formatting_with_path() -> None:
     """Verify that we cannot pass `path` for an image as a variable."""
-    in_mem = "base64mem"
+    in_mem_ = "base64mem"
 
     template = ChatPromptTemplate.from_messages(
         [
@@ -745,17 +779,29 @@ async def test_chat_tmpl_from_messages_multipart_formatting_with_path() -> None:
             ),
         ]
     )
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Loading images from 'path' has been removed as of 0.3.15 "
+            "for security reasons."
+        ),
+    ):
         template.format_messages(
             name="R2D2",
-            in_mem=in_mem,
+            in_mem=in_mem_,
             file_path="some/path",
         )
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Loading images from 'path' has been removed as of 0.3.15 "
+            "for security reasons."
+        ),
+    ):
         await template.aformat_messages(
             name="R2D2",
-            in_mem=in_mem,
+            in_mem=in_mem_,
             file_path="some/path",
         )
 
@@ -845,26 +891,29 @@ def test_chat_prompt_message_placeholder_dict() -> None:
 
 def test_chat_prompt_message_dict() -> None:
     prompt = ChatPromptTemplate(
-        [{"role": "system", "content": "foo"}, {"role": "user", "content": "bar"}]
+        [
+            {"role": "system", "content": "foo"},
+            {"role": "user", "content": "bar"},
+        ]
     )
     assert prompt.format_messages() == [
         SystemMessage(content="foo"),
         HumanMessage(content="bar"),
     ]
 
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="Invalid template: False"):
         ChatPromptTemplate([{"role": "system", "content": False}])
 
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="Unexpected message type: foo"):
         ChatPromptTemplate([{"role": "foo", "content": "foo"}])
 
 
 async def test_messages_prompt_accepts_list() -> None:
     prompt = ChatPromptTemplate([MessagesPlaceholder("history")])
-    value = prompt.invoke([("user", "Hi there")])  # type: ignore
+    value = prompt.invoke([("user", "Hi there")])  # type: ignore[arg-type]
     assert value.to_messages() == [HumanMessage(content="Hi there")]
 
-    value = await prompt.ainvoke([("user", "Hi there")])  # type: ignore
+    value = await prompt.ainvoke([("user", "Hi there")])  # type: ignore[arg-type]
     assert value.to_messages() == [HumanMessage(content="Hi there")]
 
     # Assert still raises a nice error
@@ -875,10 +924,10 @@ async def test_messages_prompt_accepts_list() -> None:
         ]
     )
     with pytest.raises(TypeError):
-        prompt.invoke([("user", "Hi there")])  # type: ignore
+        prompt.invoke([("user", "Hi there")])  # type: ignore[arg-type]
 
     with pytest.raises(TypeError):
-        await prompt.ainvoke([("user", "Hi there")])  # type: ignore
+        await prompt.ainvoke([("user", "Hi there")])  # type: ignore[arg-type]
 
 
 def test_chat_input_schema(snapshot: SnapshotAssertion) -> None:
@@ -890,7 +939,7 @@ def test_chat_input_schema(snapshot: SnapshotAssertion) -> None:
     with pytest.raises(ValidationError):
         prompt_all_required.input_schema(input="")
 
-    if (PYDANTIC_MAJOR_VERSION, PYDANTIC_MINOR_VERSION) >= (2, 10):
+    if version.parse("2.10") <= PYDANTIC_VERSION:
         assert _normalize_schema(
             prompt_all_required.get_input_jsonschema()
         ) == snapshot(name="required")
@@ -901,7 +950,7 @@ def test_chat_input_schema(snapshot: SnapshotAssertion) -> None:
     assert set(prompt_optional.input_variables) == {"input"}
     prompt_optional.input_schema(input="")  # won't raise error
 
-    if (PYDANTIC_MAJOR_VERSION, PYDANTIC_MINOR_VERSION) >= (2, 10):
+    if version.parse("2.10") <= PYDANTIC_VERSION:
         assert _normalize_schema(prompt_optional.get_input_jsonschema()) == snapshot(
             name="partial"
         )
@@ -921,7 +970,7 @@ def test_chat_prompt_w_msgs_placeholder_ser_des(snapshot: SnapshotAssertion) ->
     assert load(dumpd(prompt)) == prompt
 
 
-async def test_chat_tmpl_serdes(snapshot: SnapshotAssertion) -> None:
+def test_chat_tmpl_serdes(snapshot: SnapshotAssertion) -> None:
     """Test chat prompt template ser/des."""
     template = ChatPromptTemplate(
         [
@@ -929,13 +978,18 @@ async def test_chat_tmpl_serdes(snapshot: SnapshotAssertion) -> None:
             ("system", [{"text": "You are an AI assistant named {name}."}]),
             SystemMessagePromptTemplate.from_template("you are {foo}"),
             cast(
-                tuple,
+                "tuple",
                 (
                     "human",
                     [
                         "hello",
                         {"text": "What's in this image?"},
                         {"type": "text", "text": "What's in this image?"},
+                        {
+                            "type": "text",
+                            "text": "What's in this image?",
+                            "cache_control": {"type": "{foo}"},
+                        },
                         {
                             "type": "image_url",
                             "image_url": "data:image/jpeg;base64,{my_image}",
@@ -972,6 +1026,89 @@ async def test_chat_tmpl_serdes(snapshot: SnapshotAssertion) -> None:
     assert load(dumpd(template)) == template
 
 
+@pytest.mark.xfail(
+    reason=(
+        "In a breaking release, we can update `_convert_to_message_template` to use "
+        "DictPromptTemplate for all `dict` inputs, allowing for templatization "
+        "of message attributes outside content blocks. That would enable the below "
+        "test to pass."
+    )
+)
+def test_chat_tmpl_dict_msg() -> None:
+    template = ChatPromptTemplate(
+        [
+            {
+                "role": "assistant",
+                "content": [
+                    {
+                        "type": "text",
+                        "text": "{text1}",
+                        "cache_control": {"type": "ephemeral"},
+                    },
+                ],
+                "name": "{name1}",
+                "tool_calls": [
+                    {
+                        "name": "{tool_name1}",
+                        "args": {"arg1": "{tool_arg1}"},
+                        "id": "1",
+                        "type": "tool_call",
+                    }
+                ],
+            },
+            {
+                "role": "tool",
+                "content": "{tool_content2}",
+                "tool_call_id": "1",
+                "name": "{tool_name1}",
+            },
+        ]
+    )
+    expected = [
+        AIMessage(
+            [
+                {
+                    "type": "text",
+                    "text": "important message",
+                    "cache_control": {"type": "ephemeral"},
+                },
+            ],
+            name="foo",
+            tool_calls=[
+                {
+                    "name": "do_stuff",
+                    "args": {"arg1": "important arg1"},
+                    "id": "1",
+                    "type": "tool_call",
+                }
+            ],
+        ),
+        ToolMessage("foo", name="do_stuff", tool_call_id="1"),
+    ]
+
+    actual = template.invoke(
+        {
+            "text1": "important message",
+            "name1": "foo",
+            "tool_arg1": "important arg1",
+            "tool_name1": "do_stuff",
+            "tool_content2": "foo",
+        }
+    ).to_messages()
+    assert actual == expected
+
+    partial_ = template.partial(text1="important message")
+    actual = partial_.invoke(
+        {
+            "name1": "foo",
+            "tool_arg1": "important arg1",
+            "tool_name1": "do_stuff",
+            "tool_content2": "foo",
+        }
+    ).to_messages()
+    assert actual == expected
+
+
 def test_chat_prompt_template_variable_names() -> None:
     """This test was written for an edge case that triggers a warning from Pydantic.
 
@@ -983,13 +1120,12 @@ def test_chat_prompt_template_variable_names() -> None:
         prompt.get_input_schema()
 
     if record:
-        error_msg = []
-        for warning in record:
-            error_msg.append(
-                f"Warning type: {warning.category.__name__}, "
-                f"Warning message: {warning.message}, "
-                f"Warning location: {warning.filename}:{warning.lineno}"
-            )
+        error_msg = [
+            f"Warning type: {warning.category.__name__}, "
+            f"Warning message: {warning.message}, "
+            f"Warning location: {warning.filename}:{warning.lineno}"
+            for warning in record
+        ]
         msg = "\n".join(error_msg)
     else:
         msg = ""
@@ -1016,3 +1152,87 @@ def test_chat_prompt_template_variable_names() -> None:
         "title": "PromptInput",
         "type": "object",
     }
+
+
+def test_data_prompt_template_deserializable() -> None:
+    """Test that the image prompt template is serializable."""
+    load(
+        dumpd(
+            ChatPromptTemplate.from_messages(
+                [
+                    (
+                        "system",
+                        [{"type": "image", "source_type": "url", "url": "{url}"}],
+                    )
+                ]
+            )
+        )
+    )
+
+
+@pytest.mark.requires("jinja2")
+@pytest.mark.parametrize(
+    ("template_format", "cache_control_placeholder", "source_data_placeholder"),
+    [
+        ("f-string", "{cache_type}", "{source_data}"),
+        ("mustache", "{{cache_type}}", "{{source_data}}"),
+    ],
+)
+def test_chat_prompt_template_data_prompt_from_message(
+    template_format: PromptTemplateFormat,
+    cache_control_placeholder: str,
+    source_data_placeholder: str,
+) -> None:
+    prompt: dict = {
+        "type": "image",
+        "source_type": "base64",
+        "data": f"{source_data_placeholder}",
+    }
+
+    template = ChatPromptTemplate.from_messages(
+        [("human", [prompt])], template_format=template_format
+    )
+    assert template.format_messages(source_data="base64data") == [
+        HumanMessage(
+            content=[
+                {
+                    "type": "image",
+                    "source_type": "base64",
+                    "data": "base64data",
+                }
+            ]
+        )
+    ]
+
+    # metadata
+    prompt["metadata"] = {"cache_control": {"type": f"{cache_control_placeholder}"}}
+    template = ChatPromptTemplate.from_messages(
+        [("human", [prompt])], template_format=template_format
+    )
+    assert template.format_messages(
+        cache_type="ephemeral", source_data="base64data"
+    ) == [
+        HumanMessage(
+            content=[
+                {
+                    "type": "image",
+                    "source_type": "base64",
+                    "data": "base64data",
+                    "metadata": {"cache_control": {"type": "ephemeral"}},
+                }
+            ]
+        )
+    ]
+
+
+def test_dict_message_prompt_template_errors_on_jinja2() -> None:
+    prompt = {
+        "type": "image",
+        "source_type": "base64",
+        "data": "{source_data}",
+    }
+
+    with pytest.raises(ValueError, match="jinja2"):
+        _ = ChatPromptTemplate.from_messages(
+            [("human", [prompt])], template_format="jinja2"
+        )
