@@ -1,18 +1,21 @@
 """Test functionality related to prompts."""
 
-from typing import Any, Union
+import re
+from tempfile import NamedTemporaryFile
+from typing import Any, Literal, Union
 from unittest import mock
 
-import pydantic
 import pytest
-from syrupy import SnapshotAssertion
+from packaging import version
+from syrupy.assertion import SnapshotAssertion
 
 from langchain_core.prompts.prompt import PromptTemplate
 from langchain_core.prompts.string import PromptTemplateFormat
 from langchain_core.tracers.run_collector import RunCollectorCallbackHandler
+from langchain_core.utils.pydantic import PYDANTIC_VERSION
 from tests.unit_tests.pydantic_utils import _normalize_schema
 
-PYDANTIC_VERSION = tuple(map(int, pydantic.__version__.split(".")))
+PYDANTIC_VERSION_AT_LEAST_29 = version.parse("2.9") <= PYDANTIC_VERSION
 
 
 def test_prompt_valid() -> None:
@@ -30,8 +33,6 @@ def test_from_file_encoding() -> None:
     input_variables = ["foo"]
 
     # First write to a file using CP-1252 encoding.
-    from tempfile import NamedTemporaryFile
-
     with NamedTemporaryFile(delete=True, mode="w", encoding="cp1252") as f:
         f.write(template)
         f.flush()
@@ -116,7 +117,7 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
         "This foo is a bar test baz."
     )
     assert prompt.input_variables == ["foo", "obj"]
-    if PYDANTIC_VERSION >= (2, 9):
+    if PYDANTIC_VERSION_AT_LEAST_29:
         assert _normalize_schema(prompt.get_input_jsonschema()) == snapshot(
             name="schema_0"
         )
@@ -143,7 +144,7 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
     is a test."""
     )
     assert prompt.input_variables == ["foo"]
-    if PYDANTIC_VERSION >= (2, 9):
+    if PYDANTIC_VERSION_AT_LEAST_29:
         assert _normalize_schema(prompt.get_input_jsonschema()) == snapshot(
             name="schema_2"
         )
@@ -167,7 +168,7 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
     is a test."""
     )
     assert prompt.input_variables == ["foo"]
-    if PYDANTIC_VERSION >= (2, 9):
+    if PYDANTIC_VERSION_AT_LEAST_29:
         assert _normalize_schema(prompt.get_input_jsonschema()) == snapshot(
             name="schema_3"
         )
@@ -205,7 +206,7 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
     is a test."""
     )
     assert prompt.input_variables == ["foo"]
-    if PYDANTIC_VERSION >= (2, 9):
+    if PYDANTIC_VERSION_AT_LEAST_29:
         assert _normalize_schema(prompt.get_input_jsonschema()) == snapshot(
             name="schema_4"
         )
@@ -223,7 +224,7 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
     is a test."""  # noqa: W293
     )
     assert prompt.input_variables == ["foo"]
-    if PYDANTIC_VERSION >= (2, 9):
+    if PYDANTIC_VERSION_AT_LEAST_29:
         assert _normalize_schema(prompt.get_input_jsonschema()) == snapshot(
             name="schema_5"
         )
@@ -237,8 +238,8 @@ def test_mustache_prompt_from_template(snapshot: SnapshotAssertion) -> None:
     is a test."""
     )
     assert prompt.input_variables == ["foo"]
-    assert prompt.get_input_jsonschema() == {
-        "properties": {"foo": {"default": None, "title": "Foo", "type": "object"}},
+    assert _normalize_schema(prompt.get_input_jsonschema()) == {
+        "properties": {"foo": {"title": "Foo", "type": "object"}},
         "title": "PromptInput",
         "type": "object",
     }
@@ -264,7 +265,10 @@ def test_prompt_missing_input_variables() -> None:
     """Test error is raised when input variables are not provided."""
     template = "This is a {foo} test."
     input_variables: list = []
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape("check for mismatched or missing input parameters from []"),
+    ):
         PromptTemplate(
             input_variables=input_variables, template=template, validate_template=True
         )
@@ -275,7 +279,10 @@ def test_prompt_missing_input_variables() -> None:
 
 def test_prompt_empty_input_variable() -> None:
     """Test error is raised when empty string input variable."""
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape("check for mismatched or missing input parameters from ['']"),
+    ):
         PromptTemplate(input_variables=[""], template="{}", validate_template=True)
 
 
@@ -283,7 +290,13 @@ def test_prompt_wrong_input_variables() -> None:
     """Test error is raised when name of input variable is wrong."""
     template = "This is a {foo} test."
     input_variables = ["bar"]
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Invalid prompt schema; "
+            "check for mismatched or missing input parameters from ['bar']"
+        ),
+    ):
         PromptTemplate(
             input_variables=input_variables, template=template, validate_template=True
         )
@@ -330,25 +343,25 @@ def test_prompt_invalid_template_format() -> None:
     """Test initializing a prompt with invalid template format."""
     template = "This is a {foo} test."
     input_variables = ["foo"]
-    with pytest.raises(ValueError):
+    with pytest.raises(ValueError, match="Unsupported template format: bar"):
         PromptTemplate(
             input_variables=input_variables,
             template=template,
-            template_format="bar",  # type: ignore[arg-type]
+            template_format="bar",
         )
 
 
 def test_prompt_from_file() -> None:
     """Test prompt can be successfully constructed from a file."""
     template_file = "tests/unit_tests/data/prompt_file.txt"
-    input_variables = ["question"]
-    prompt = PromptTemplate.from_file(template_file, input_variables)
+    prompt = PromptTemplate.from_file(template_file)
     assert prompt.template == "Question: {question}\nAnswer:"
 
 
 def test_prompt_from_file_with_partial_variables() -> None:
-    """Test prompt can be successfully constructed from a file
-    with partial variables.
+    """Test prompt from file with partial variables.
+
+    Test prompt can be successfully constructed from a file with partial variables.
     """
     # given
     template = "This is a {foo} test {bar}."
@@ -419,15 +432,13 @@ Will it get confused{ }?
     assert prompt == expected_prompt
 
 
-@pytest.mark.requires("jinja2")
 def test_basic_sandboxing_with_jinja2() -> None:
     """Test basic sandboxing with jinja2."""
-    import jinja2
-
+    jinja2 = pytest.importorskip("jinja2")
     template = " {{''.__class__.__bases__[0] }} "  # malicious code
     prompt = PromptTemplate.from_template(template, template_format="jinja2")
     with pytest.raises(jinja2.exceptions.SecurityError):
-        assert prompt.format() == []
+        prompt.format()
 
 
 @pytest.mark.requires("jinja2")
@@ -495,7 +506,7 @@ def test_prompt_jinja2_missing_input_variables() -> None:
     """Test error is raised when input variables are not provided."""
     template = "This is a {{ foo }} test."
     input_variables: list = []
-    with pytest.warns(UserWarning):
+    with pytest.warns(UserWarning, match="Missing variables: {'foo'}"):
         PromptTemplate(
             input_variables=input_variables,
             template=template,
@@ -512,7 +523,7 @@ def test_prompt_jinja2_extra_input_variables() -> None:
     """Test error is raised when there are too many input variables."""
     template = "This is a {{ foo }} test."
     input_variables = ["foo", "bar"]
-    with pytest.warns(UserWarning):
+    with pytest.warns(UserWarning, match="Extra variables: {'bar'}"):
         PromptTemplate(
             input_variables=input_variables,
             template=template,
@@ -529,7 +540,9 @@ def test_prompt_jinja2_wrong_input_variables() -> None:
     """Test error is raised when name of input variable is wrong."""
     template = "This is a {{ foo }} test."
     input_variables = ["bar"]
-    with pytest.warns(UserWarning):
+    with pytest.warns(
+        UserWarning, match="Missing variables: {'foo'} Extra variables: {'bar'}"
+    ):
         PromptTemplate(
             input_variables=input_variables,
             template=template,
@@ -556,8 +569,8 @@ def test_prompt_invoke_with_metadata() -> None:
     )
     assert result.to_string() == "This is a bar test."
     assert len(tracer.traced_runs) == 1
-    assert tracer.traced_runs[0].extra["metadata"] == {"version": "1", "foo": "bar"}  # type: ignore
-    assert tracer.traced_runs[0].tags == ["tag1", "tag2"]  # type: ignore
+    assert tracer.traced_runs[0].extra["metadata"] == {"version": "1", "foo": "bar"}
+    assert tracer.traced_runs[0].tags == ["tag1", "tag2"]
 
 
 async def test_prompt_ainvoke_with_metadata() -> None:
@@ -575,12 +588,12 @@ async def test_prompt_ainvoke_with_metadata() -> None:
     )
     assert result.to_string() == "This is a bar test."
     assert len(tracer.traced_runs) == 1
-    assert tracer.traced_runs[0].extra["metadata"] == {"version": "1", "foo": "bar"}  # type: ignore
-    assert tracer.traced_runs[0].tags == ["tag1", "tag2"]  # type: ignore
+    assert tracer.traced_runs[0].extra["metadata"] == {"version": "1", "foo": "bar"}
+    assert tracer.traced_runs[0].tags == ["tag1", "tag2"]
 
 
 @pytest.mark.parametrize(
-    "value, expected",
+    ("value", "expected"),
     [
         ("0", "0"),
         (0, "0"),
@@ -664,3 +677,54 @@ def test_prompt_with_template_variable_name_jinja2() -> None:
     template = "This is a {{template}} test."
     prompt = PromptTemplate.from_template(template, template_format="jinja2")
     assert prompt.invoke({"template": "bar"}).to_string() == "This is a bar test."
+
+
+def test_prompt_template_add_with_with_another_format() -> None:
+    with pytest.raises(ValueError, match=r"Cannot add templates"):
+        (
+            PromptTemplate.from_template("This is a {template}")
+            + PromptTemplate.from_template("So {{this}} is", template_format="mustache")
+        )
+
+
+@pytest.mark.parametrize(
+    ("template_format", "prompt1", "prompt2"),
+    [
+        ("f-string", "This is a {variable}", ". This is {another_variable}"),
+        pytest.param(
+            "jinja2",
+            "This is a {{variable}}",
+            ". This is {{another_variable}}",
+            marks=[pytest.mark.requires("jinja2")],
+        ),
+        ("mustache", "This is a {{variable}}", ". This is {{another_variable}}"),
+    ],
+)
+def test_prompt_template_add(
+    template_format: Literal["f-string", "mustache", "jinja2"],
+    prompt1: str,
+    prompt2: str,
+) -> None:
+    first_prompt = PromptTemplate.from_template(
+        prompt1,
+        template_format=template_format,
+    )
+    second_prompt = PromptTemplate.from_template(
+        prompt2,
+        template_format=template_format,
+    )
+
+    concated_prompt = first_prompt + second_prompt
+    prompt_of_concated = PromptTemplate.from_template(
+        prompt1 + prompt2,
+        template_format=template_format,
+    )
+
+    assert concated_prompt.input_variables == prompt_of_concated.input_variables
+    assert concated_prompt.format(
+        variable="template",
+        another_variable="other_template",
+    ) == prompt_of_concated.format(
+        variable="template",
+        another_variable="other_template",
+    )
