@@ -1,8 +1,11 @@
+import re
 from collections.abc import Sequence
 from typing import Any, Callable, Optional, Union
 
 import pytest
-from pydantic import BaseModel
+from packaging import version
+from pydantic import BaseModel, RootModel
+from typing_extensions import override
 
 from langchain_core.callbacks import (
     CallbackManagerForLLMRun,
@@ -17,6 +20,12 @@ from langchain_core.runnables.config import RunnableConfig
 from langchain_core.runnables.history import RunnableWithMessageHistory
 from langchain_core.runnables.utils import ConfigurableFieldSpec, Input, Output
 from langchain_core.tracers import Run
+from langchain_core.tracers.root_listeners import (
+    AsyncListener,
+    AsyncRootListenersTracer,
+    RootListenersTracer,
+)
+from langchain_core.utils.pydantic import PYDANTIC_VERSION
 from tests.unit_tests.pydantic_utils import _schema
 
 
@@ -36,7 +45,7 @@ def _get_get_session_history(
     chat_history_store = store if store is not None else {}
 
     def get_session_history(
-        session_id: str, **kwargs: Any
+        session_id: str, **_kwargs: Any
     ) -> InMemoryChatMessageHistory:
         if session_id not in chat_history_store:
             chat_history_store[session_id] = InMemoryChatMessageHistory()
@@ -107,9 +116,9 @@ async def test_input_messages_async() -> None:
 
 def test_input_dict() -> None:
     runnable = RunnableLambda(
-        lambda input: "you said: "
+        lambda params: "you said: "
         + "\n".join(
-            str(m.content) for m in input["messages"] if isinstance(m, HumanMessage)
+            str(m.content) for m in params["messages"] if isinstance(m, HumanMessage)
         )
     )
     get_session_history = _get_get_session_history()
@@ -127,9 +136,9 @@ def test_input_dict() -> None:
 
 async def test_input_dict_async() -> None:
     runnable = RunnableLambda(
-        lambda input: "you said: "
+        lambda params: "you said: "
         + "\n".join(
-            str(m.content) for m in input["messages"] if isinstance(m, HumanMessage)
+            str(m.content) for m in params["messages"] if isinstance(m, HumanMessage)
         )
     )
     get_session_history = _get_get_session_history()
@@ -149,10 +158,10 @@ async def test_input_dict_async() -> None:
 
 def test_input_dict_with_history_key() -> None:
     runnable = RunnableLambda(
-        lambda input: "you said: "
+        lambda params: "you said: "
         + "\n".join(
-            [str(m.content) for m in input["history"] if isinstance(m, HumanMessage)]
-            + [input["input"]]
+            [str(m.content) for m in params["history"] if isinstance(m, HumanMessage)]
+            + [params["input"]]
         )
     )
     get_session_history = _get_get_session_history()
@@ -171,10 +180,10 @@ def test_input_dict_with_history_key() -> None:
 
 async def test_input_dict_with_history_key_async() -> None:
     runnable = RunnableLambda(
-        lambda input: "you said: "
+        lambda params: "you said: "
         + "\n".join(
-            [str(m.content) for m in input["history"] if isinstance(m, HumanMessage)]
-            + [input["input"]]
+            [str(m.content) for m in params["history"] if isinstance(m, HumanMessage)]
+            + [params["input"]]
         )
     )
     get_session_history = _get_get_session_history()
@@ -193,15 +202,15 @@ async def test_input_dict_with_history_key_async() -> None:
 
 def test_output_message() -> None:
     runnable = RunnableLambda(
-        lambda input: AIMessage(
+        lambda params: AIMessage(
             content="you said: "
             + "\n".join(
                 [
                     str(m.content)
-                    for m in input["history"]
+                    for m in params["history"]
                     if isinstance(m, HumanMessage)
                 ]
-                + [input["input"]]
+                + [params["input"]]
             )
         )
     )
@@ -221,15 +230,15 @@ def test_output_message() -> None:
 
 async def test_output_message_async() -> None:
     runnable = RunnableLambda(
-        lambda input: AIMessage(
+        lambda params: AIMessage(
             content="you said: "
             + "\n".join(
                 [
                     str(m.content)
-                    for m in input["history"]
+                    for m in params["history"]
                     if isinstance(m, HumanMessage)
                 ]
-                + [input["input"]]
+                + [params["input"]]
             )
         )
     )
@@ -250,6 +259,7 @@ async def test_output_message_async() -> None:
 class LengthChatModel(BaseChatModel):
     """A fake chat model that returns the length of the messages passed in."""
 
+    @override
     def _generate(
         self,
         messages: list[BaseMessage],
@@ -297,23 +307,23 @@ async def test_input_messages_output_message_async() -> None:
 
 def test_output_messages() -> None:
     runnable = RunnableLambda(
-        lambda input: [
+        lambda params: [
             AIMessage(
                 content="you said: "
                 + "\n".join(
                     [
                         str(m.content)
-                        for m in input["history"]
+                        for m in params["history"]
                         if isinstance(m, HumanMessage)
                     ]
-                    + [input["input"]]
+                    + [params["input"]]
                 )
             )
         ]
     )
     get_session_history = _get_get_session_history()
     with_history = RunnableWithMessageHistory(
-        runnable,  # type: ignore
+        runnable,
         get_session_history,
         input_messages_key="input",
         history_messages_key="history",
@@ -327,23 +337,23 @@ def test_output_messages() -> None:
 
 async def test_output_messages_async() -> None:
     runnable = RunnableLambda(
-        lambda input: [
+        lambda params: [
             AIMessage(
                 content="you said: "
                 + "\n".join(
                     [
                         str(m.content)
-                        for m in input["history"]
+                        for m in params["history"]
                         if isinstance(m, HumanMessage)
                     ]
-                    + [input["input"]]
+                    + [params["input"]]
                 )
             )
         ]
     )
     get_session_history = _get_get_session_history()
     with_history = RunnableWithMessageHistory(
-        runnable,  # type: ignore
+        runnable,
         get_session_history,
         input_messages_key="input",
         history_messages_key="history",
@@ -357,17 +367,17 @@ async def test_output_messages_async() -> None:
 
 def test_output_dict() -> None:
     runnable = RunnableLambda(
-        lambda input: {
+        lambda params: {
             "output": [
                 AIMessage(
                     content="you said: "
                     + "\n".join(
                         [
                             str(m.content)
-                            for m in input["history"]
+                            for m in params["history"]
                             if isinstance(m, HumanMessage)
                         ]
-                        + [input["input"]]
+                        + [params["input"]]
                     )
                 )
             ]
@@ -390,17 +400,17 @@ def test_output_dict() -> None:
 
 async def test_output_dict_async() -> None:
     runnable = RunnableLambda(
-        lambda input: {
+        lambda params: {
             "output": [
                 AIMessage(
                     content="you said: "
                     + "\n".join(
                         [
                             str(m.content)
-                            for m in input["history"]
+                            for m in params["history"]
                             if isinstance(m, HumanMessage)
                         ]
-                        + [input["input"]]
+                        + [params["input"]]
                     )
                 )
             ]
@@ -426,17 +436,17 @@ def test_get_input_schema_input_dict() -> None:
         input: Union[str, BaseMessage, Sequence[BaseMessage]]
 
     runnable = RunnableLambda(
-        lambda input: {
+        lambda params: {
             "output": [
                 AIMessage(
                     content="you said: "
                     + "\n".join(
                         [
                             str(m.content)
-                            for m in input["history"]
+                            for m in params["history"]
                             if isinstance(m, HumanMessage)
                         ]
-                        + [input["input"]]
+                        + [params["input"]]
                     )
                 )
             ]
@@ -458,17 +468,17 @@ def test_get_input_schema_input_dict() -> None:
 def test_get_output_schema() -> None:
     """Test get output schema."""
     runnable = RunnableLambda(
-        lambda input: {
+        lambda params: {
             "output": [
                 AIMessage(
                     content="you said: "
                     + "\n".join(
                         [
                             str(m.content)
-                            for m in input["history"]
+                            for m in params["history"]
                             if isinstance(m, HumanMessage)
                         ]
-                        + [input["input"]]
+                        + [params["input"]]
                     )
                 )
             ]
@@ -484,15 +494,16 @@ def test_get_output_schema() -> None:
     )
     output_type = with_history.get_output_schema()
 
-    assert _schema(output_type) == {
+    expected_schema: dict = {
         "title": "RunnableWithChatHistoryOutput",
         "type": "object",
     }
+    if version.parse("2.11") <= PYDANTIC_VERSION:
+        expected_schema["additionalProperties"] = True
+    assert _schema(output_type) == expected_schema
 
 
 def test_get_input_schema_input_messages() -> None:
-    from pydantic import RootModel
-
     runnable_with_message_history_input = RootModel[Sequence[BaseMessage]]
 
     runnable = RunnableLambda(
@@ -523,8 +534,8 @@ def test_get_input_schema_input_messages() -> None:
 def test_using_custom_config_specs() -> None:
     """Test that we can configure which keys should be passed to the session factory."""
 
-    def _fake_llm(input: dict[str, Any]) -> list[BaseMessage]:
-        messages = input["messages"]
+    def _fake_llm(params: dict[str, Any]) -> list[BaseMessage]:
+        messages = params["messages"]
         return [
             AIMessage(
                 content="you said: "
@@ -541,11 +552,11 @@ def test_using_custom_config_specs() -> None:
         user_id: str, conversation_id: str
     ) -> InMemoryChatMessageHistory:
         if (user_id, conversation_id) not in store:
-            store[(user_id, conversation_id)] = InMemoryChatMessageHistory()
-        return store[(user_id, conversation_id)]
+            store[user_id, conversation_id] = InMemoryChatMessageHistory()
+        return store[user_id, conversation_id]
 
     with_message_history = RunnableWithMessageHistory(
-        runnable,  # type: ignore
+        runnable,
         get_session_history=get_session_history,
         input_messages_key="messages",
         history_messages_key="history",
@@ -636,8 +647,8 @@ def test_using_custom_config_specs() -> None:
 async def test_using_custom_config_specs_async() -> None:
     """Test that we can configure which keys should be passed to the session factory."""
 
-    def _fake_llm(input: dict[str, Any]) -> list[BaseMessage]:
-        messages = input["messages"]
+    def _fake_llm(params: dict[str, Any]) -> list[BaseMessage]:
+        messages = params["messages"]
         return [
             AIMessage(
                 content="you said: "
@@ -654,11 +665,11 @@ async def test_using_custom_config_specs_async() -> None:
         user_id: str, conversation_id: str
     ) -> InMemoryChatMessageHistory:
         if (user_id, conversation_id) not in store:
-            store[(user_id, conversation_id)] = InMemoryChatMessageHistory()
-        return store[(user_id, conversation_id)]
+            store[user_id, conversation_id] = InMemoryChatMessageHistory()
+        return store[user_id, conversation_id]
 
     with_message_history = RunnableWithMessageHistory(
-        runnable,  # type: ignore
+        runnable,
         get_session_history=get_session_history,
         input_messages_key="messages",
         history_messages_key="history",
@@ -749,27 +760,25 @@ async def test_using_custom_config_specs_async() -> None:
 def test_ignore_session_id() -> None:
     """Test without config."""
 
-    def _fake_llm(input: list[BaseMessage]) -> list[BaseMessage]:
+    def _fake_llm(messages: list[BaseMessage]) -> list[BaseMessage]:
         return [
             AIMessage(
                 content="you said: "
                 + "\n".join(
-                    str(m.content) for m in input if isinstance(m, HumanMessage)
+                    str(m.content) for m in messages if isinstance(m, HumanMessage)
                 )
             )
         ]
 
     runnable = RunnableLambda(_fake_llm)
     history = InMemoryChatMessageHistory()
-    with_message_history = RunnableWithMessageHistory(runnable, lambda: history)  # type: ignore
+    with_message_history = RunnableWithMessageHistory(runnable, lambda: history)
     _ = with_message_history.invoke("hello")
     _ = with_message_history.invoke("hello again")
     assert len(history.messages) == 4
 
 
-class _RunnableLambdaWithRaiseError(RunnableLambda):
-    from langchain_core.tracers.root_listeners import AsyncListener
-
+class _RunnableLambdaWithRaiseError(RunnableLambda[Input, Output]):
     def with_listeners(
         self,
         *,
@@ -783,8 +792,6 @@ class _RunnableLambdaWithRaiseError(RunnableLambda):
             Union[Callable[[Run], None], Callable[[Run, RunnableConfig], None]]
         ] = None,
     ) -> Runnable[Input, Output]:
-        from langchain_core.tracers.root_listeners import RootListenersTracer
-
         def create_tracer(config: RunnableConfig) -> RunnableConfig:
             tracer = RootListenersTracer(
                 config=config,
@@ -799,7 +806,7 @@ class _RunnableLambdaWithRaiseError(RunnableLambda):
 
         return RunnableBinding(
             bound=self,
-            config_factories=[lambda config: create_tracer(config)],
+            config_factories=[create_tracer],
         )
 
     def with_alisteners(
@@ -809,8 +816,6 @@ class _RunnableLambdaWithRaiseError(RunnableLambda):
         on_end: Optional[AsyncListener] = None,
         on_error: Optional[AsyncListener] = None,
     ) -> Runnable[Input, Output]:
-        from langchain_core.tracers.root_listeners import AsyncRootListenersTracer
-
         def create_tracer(config: RunnableConfig) -> RunnableConfig:
             tracer = AsyncRootListenersTracer(
                 config=config,
@@ -825,7 +830,7 @@ class _RunnableLambdaWithRaiseError(RunnableLambda):
 
         return RunnableBinding(
             bound=self,
-            config_factories=[lambda config: create_tracer(config)],
+            config_factories=[create_tracer],
         )
 
 
@@ -850,30 +855,32 @@ def test_get_output_messages_no_value_error() -> None:
 
 def test_get_output_messages_with_value_error() -> None:
     illegal_bool_message = False
-    runnable = _RunnableLambdaWithRaiseError(lambda messages: illegal_bool_message)
+    runnable = _RunnableLambdaWithRaiseError(lambda _: illegal_bool_message)
     store: dict = {}
     get_session_history = _get_get_session_history(store=store)
-    with_history = RunnableWithMessageHistory(runnable, get_session_history)
+    with_history = RunnableWithMessageHistory(runnable, get_session_history)  # type: ignore[arg-type]
     config: RunnableConfig = {
         "configurable": {"session_id": "1", "message_history": get_session_history("1")}
     }
 
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Expected str, BaseMessage, list[BaseMessage], or tuple[BaseMessage]."
+            f" Got {illegal_bool_message}."
+        ),
+    ):
         with_history.bound.invoke([HumanMessage(content="hello")], config)
-    excepted = (
-        "Expected str, BaseMessage, List[BaseMessage], or Tuple[BaseMessage]."
-        + (f" Got {illegal_bool_message}.")
-    )
-    assert excepted in str(excinfo.value)
 
     illegal_int_message = 123
-    runnable = _RunnableLambdaWithRaiseError(lambda messages: illegal_int_message)
-    with_history = RunnableWithMessageHistory(runnable, get_session_history)
+    runnable2 = _RunnableLambdaWithRaiseError(lambda _: illegal_int_message)
+    with_history = RunnableWithMessageHistory(runnable2, get_session_history)  # type: ignore[arg-type]
 
-    with pytest.raises(ValueError) as excinfo:
+    with pytest.raises(
+        ValueError,
+        match=re.escape(
+            "Expected str, BaseMessage, list[BaseMessage], or tuple[BaseMessage]."
+            f" Got {illegal_int_message}."
+        ),
+    ):
         with_history.bound.invoke([HumanMessage(content="hello")], config)
-    excepted = (
-        "Expected str, BaseMessage, List[BaseMessage], or Tuple[BaseMessage]."
-        + (f" Got {illegal_int_message}.")
-    )
-    assert excepted in str(excinfo.value)
