@@ -17,8 +17,10 @@ from typing import (
 import pytest
 from blockbuster import BlockBuster
 from pydantic import BaseModel
+from typing_extensions import override
 
 from langchain_core.callbacks import CallbackManagerForRetrieverRun, Callbacks
+from langchain_core.callbacks.manager import adispatch_custom_event
 from langchain_core.chat_history import BaseChatMessageHistory
 from langchain_core.documents import Document
 from langchain_core.language_models import FakeStreamingListLLM, GenericFakeChatModel
@@ -68,7 +70,7 @@ def _with_nulled_run_id(events: Sequence[StreamEvent]) -> list[StreamEvent]:
         )
 
     return cast(
-        list[StreamEvent],
+        "list[StreamEvent]",
         [{**event, "run_id": "", "parent_ids": []} for event in events],
     )
 
@@ -80,7 +82,7 @@ async def _as_async_iterator(iterable: list) -> AsyncIterator:
 
 
 async def _collect_events(
-    events: AsyncIterator[StreamEvent], with_nulled_ids: bool = True
+    events: AsyncIterator[StreamEvent], *, with_nulled_ids: bool = True
 ) -> list[StreamEvent]:
     """Collect the events and remove the run ids."""
     materialized_events = [event async for event in events]
@@ -97,12 +99,12 @@ async def _collect_events(
 async def test_event_stream_with_simple_function_tool() -> None:
     """Test the event stream with a function and tool."""
 
-    def foo(x: int) -> dict:
+    def foo(x: int) -> dict:  # noqa: ARG001
         """Foo."""
         return {"x": 5}
 
     @tool
-    def get_docs(x: int) -> list[Document]:
+    def get_docs(x: int) -> list[Document]:  # noqa: ARG001
         """Hello Doc."""
         return [Document(page_content="hello")]
 
@@ -465,7 +467,7 @@ async def test_event_stream_with_triple_lambda_test_filtering() -> None:
 
 
 async def test_event_stream_with_lambdas_from_lambda() -> None:
-    as_lambdas = RunnableLambda(lambda x: {"answer": "goodbye"}).with_config(
+    as_lambdas = RunnableLambda(lambda _: {"answer": "goodbye"}).with_config(
         {"run_name": "my_lambda"}
     )
     events = await _collect_events(
@@ -612,11 +614,10 @@ async def test_astream_with_model_in_chain() -> None:
     )
 
     @RunnableLambda
-    def i_dont_stream(input: Any, config: RunnableConfig) -> Any:
+    def i_dont_stream(value: Any, config: RunnableConfig) -> Any:
         if sys.version_info >= (3, 11):
-            return model.invoke(input)
-        else:
-            return model.invoke(input, config)
+            return model.invoke(value)
+        return model.invoke(value, config)
 
     events = await _collect_events(i_dont_stream.astream_events("hello", version="v2"))
     _assert_events_equal_allow_superset_metadata(
@@ -721,11 +722,10 @@ async def test_astream_with_model_in_chain() -> None:
     )
 
     @RunnableLambda
-    async def ai_dont_stream(input: Any, config: RunnableConfig) -> Any:
+    async def ai_dont_stream(value: Any, config: RunnableConfig) -> Any:
         if sys.version_info >= (3, 11):
-            return await model.ainvoke(input)
-        else:
-            return await model.ainvoke(input, config)
+            return await model.ainvoke(value)
+        return await model.ainvoke(value, config)
 
     events = await _collect_events(ai_dont_stream.astream_events("hello", version="v2"))
     _assert_events_equal_allow_superset_metadata(
@@ -1045,7 +1045,7 @@ async def test_event_streaming_with_tools() -> None:
         return "hello"
 
     @tool
-    def with_callbacks(callbacks: Callbacks) -> str:
+    def with_callbacks(callbacks: Callbacks) -> str:  # noqa: ARG001
         """A tool that does nothing."""
         return "world"
 
@@ -1055,13 +1055,13 @@ async def test_event_streaming_with_tools() -> None:
         return {"x": x, "y": y}
 
     @tool
-    def with_parameters_and_callbacks(x: int, y: str, callbacks: Callbacks) -> dict:
+    def with_parameters_and_callbacks(x: int, y: str, callbacks: Callbacks) -> dict:  # noqa: ARG001
         """A tool that does nothing."""
         return {"x": x, "y": y}
 
     # type ignores below because the tools don't appear to be runnables to type checkers
     # we can remove as soon as that's fixed
-    events = await _collect_events(parameterless.astream_events({}, version="v2"))  # type: ignore
+    events = await _collect_events(parameterless.astream_events({}, version="v2"))
     _assert_events_equal_allow_superset_metadata(
         events,
         [
@@ -1085,7 +1085,7 @@ async def test_event_streaming_with_tools() -> None:
             },
         ],
     )
-    events = await _collect_events(with_callbacks.astream_events({}, version="v2"))  # type: ignore
+    events = await _collect_events(with_callbacks.astream_events({}, version="v2"))
     _assert_events_equal_allow_superset_metadata(
         events,
         [
@@ -1110,7 +1110,7 @@ async def test_event_streaming_with_tools() -> None:
         ],
     )
     events = await _collect_events(
-        with_parameters.astream_events({"x": 1, "y": "2"}, version="v2")  # type: ignore
+        with_parameters.astream_events({"x": 1, "y": "2"}, version="v2")
     )
     _assert_events_equal_allow_superset_metadata(
         events,
@@ -1137,7 +1137,7 @@ async def test_event_streaming_with_tools() -> None:
     )
 
     events = await _collect_events(
-        with_parameters_and_callbacks.astream_events({"x": 1, "y": "2"}, version="v2")  # type: ignore
+        with_parameters_and_callbacks.astream_events({"x": 1, "y": "2"}, version="v2")
     )
     _assert_events_equal_allow_superset_metadata(
         events,
@@ -1167,6 +1167,7 @@ async def test_event_streaming_with_tools() -> None:
 class HardCodedRetriever(BaseRetriever):
     documents: list[Document]
 
+    @override
     def _get_relevant_documents(
         self, query: str, *, run_manager: CallbackManagerForRetrieverRun
     ) -> list[Document]:
@@ -1354,7 +1355,7 @@ async def test_event_stream_on_chain_with_tool() -> None:
 
     # For whatever reason type annotations fail here because reverse
     # does not appear to be a runnable
-    chain = concat | reverse  # type: ignore
+    chain = concat | reverse
 
     events = await _collect_events(
         chain.astream_events({"a": "hello", "b": "world"}, version="v2")
@@ -1453,12 +1454,12 @@ async def test_chain_ordering() -> None:
 
     events = []
 
-    for _ in range(10):
-        try:
+    try:
+        for _ in range(10):
             next_chunk = await iterable.__anext__()
             events.append(next_chunk)
-        except Exception:
-            break
+    except Exception:
+        pass
 
     events = _with_nulled_run_id(events)
     for event in events:
@@ -1555,10 +1556,10 @@ async def test_chain_ordering() -> None:
 async def test_event_stream_with_retry() -> None:
     """Test the event stream with a tool."""
 
-    def success(inputs: str) -> str:
+    def success(_: str) -> str:
         return "success"
 
-    def fail(inputs: str) -> None:
+    def fail(_: str) -> None:
         """Simple func."""
         msg = "fail"
         raise ValueError(msg)
@@ -1570,12 +1571,12 @@ async def test_event_stream_with_retry() -> None:
 
     events = []
 
-    for _ in range(10):
-        try:
+    try:
+        for _ in range(10):
             next_chunk = await iterable.__anext__()
             events.append(next_chunk)
-        except Exception:
-            break
+    except Exception:
+        pass
 
     events = _with_nulled_run_id(events)
     for event in events:
@@ -1769,12 +1770,11 @@ async def test_runnable_each() -> None:
     async def add_one(x: int) -> int:
         return x + 1
 
-    add_one_map = RunnableLambda(add_one).map()  # type: ignore
+    add_one_map = RunnableLambda(add_one).map()  # type: ignore[arg-type,var-annotated]
     assert await add_one_map.ainvoke([1, 2, 3]) == [2, 3, 4]
 
     with pytest.raises(NotImplementedError):
-        async for _ in add_one_map.astream_events([1, 2, 3], version="v2"):
-            pass
+        _ = [_ async for _ in add_one_map.astream_events([1, 2, 3], version="v2")]
 
 
 async def test_events_astream_config() -> None:
@@ -1855,7 +1855,7 @@ async def test_runnable_with_message_history() -> None:
         # where it re-instantiates a list, so mutating the list doesn't end up mutating
         # the content in the store!
 
-        # Using Any type here rather than List[BaseMessage] due to pydantic issue!
+        # Using Any type here rather than list[BaseMessage] due to pydantic issue!
         messages: Any
 
         def add_message(self, message: BaseMessage) -> None:
@@ -1891,7 +1891,7 @@ async def test_runnable_with_message_history() -> None:
     )
     model = GenericFakeChatModel(messages=infinite_cycle)
 
-    chain: Runnable = prompt | model
+    chain = prompt | model
     with_message_history = RunnableWithMessageHistory(
         chain,
         get_session_history=get_by_session_id,
@@ -2017,7 +2017,7 @@ async def test_sync_in_async_stream_lambdas(blockbuster: BlockBuster) -> None:
         results = list(streaming)
         return results[0]
 
-    add_one_proxy_ = RunnableLambda(add_one_proxy)  # type: ignore
+    add_one_proxy_ = RunnableLambda(add_one_proxy)  # type: ignore[arg-type,var-annotated]
 
     events = await _collect_events(add_one_proxy_.astream_events(1, version="v2"))
     _assert_events_equal_allow_superset_metadata(events, EXPECTED_EVENTS)
@@ -2029,7 +2029,7 @@ async def test_async_in_async_stream_lambdas() -> None:
     async def add_one(x: int) -> int:
         return x + 1
 
-    add_one_ = RunnableLambda(add_one)  # type: ignore
+    add_one_ = RunnableLambda(add_one)  # type: ignore[arg-type,var-annotated]
 
     async def add_one_proxy(x: int, config: RunnableConfig) -> int:
         # Use sync streaming
@@ -2037,7 +2037,7 @@ async def test_async_in_async_stream_lambdas() -> None:
         results = [result async for result in streaming]
         return results[0]
 
-    add_one_proxy_ = RunnableLambda(add_one_proxy)  # type: ignore
+    add_one_proxy_ = RunnableLambda(add_one_proxy)  # type: ignore[arg-type,var-annotated]
 
     events = await _collect_events(add_one_proxy_.astream_events(1, version="v2"))
     _assert_events_equal_allow_superset_metadata(events, EXPECTED_EVENTS)
@@ -2072,6 +2072,7 @@ class StreamingRunnable(Runnable[Input, Output]):
         """Initialize the runnable."""
         self.iterable = iterable
 
+    @override
     def invoke(
         self, input: Input, config: Optional[RunnableConfig] = None, **kwargs: Any
     ) -> Output:
@@ -2079,6 +2080,7 @@ class StreamingRunnable(Runnable[Input, Output]):
         msg = "Server side error"
         raise ValueError(msg)
 
+    @override
     def stream(
         self,
         input: Input,
@@ -2087,6 +2089,7 @@ class StreamingRunnable(Runnable[Input, Output]):
     ) -> Iterator[Output]:
         raise NotImplementedError
 
+    @override
     async def astream(
         self,
         input: Input,
@@ -2189,19 +2192,19 @@ async def test_parent_run_id_assignment() -> None:
     # Type ignores in the code below need to be investigated.
     # Looks like a typing issue when using RunnableLambda as a decorator
     # with async functions.
-    @RunnableLambda  # type: ignore
+    @RunnableLambda  # type: ignore[arg-type]
     async def grandchild(x: str) -> str:
         return x
 
-    @RunnableLambda  # type: ignore
+    @RunnableLambda  # type: ignore[arg-type]
     async def child(x: str, config: RunnableConfig) -> str:
         config["run_id"] = uuid.UUID(int=9)
-        return await grandchild.ainvoke(x, config)  # type: ignore
+        return await grandchild.ainvoke(x, config)  # type: ignore[arg-type]
 
-    @RunnableLambda  # type: ignore
+    @RunnableLambda  # type: ignore[arg-type]
     async def parent(x: str, config: RunnableConfig) -> str:
         config["run_id"] = uuid.UUID(int=8)
-        return await child.ainvoke(x, config)  # type: ignore
+        return await child.ainvoke(x, config)  # type: ignore[arg-type]
 
     bond = uuid.UUID(int=7)
     events = await _collect_events(
@@ -2290,14 +2293,14 @@ async def test_bad_parent_ids() -> None:
     # Type ignores in the code below need to be investigated.
     # Looks like a typing issue when using RunnableLambda as a decorator
     # with async functions.
-    @RunnableLambda  # type: ignore
+    @RunnableLambda  # type: ignore[arg-type]
     async def child(x: str) -> str:
         return x
 
-    @RunnableLambda  # type: ignore
+    @RunnableLambda  # type: ignore[arg-type]
     async def parent(x: str, config: RunnableConfig) -> str:
         config["run_id"] = uuid.UUID(int=7)
-        return await child.ainvoke(x, config)  # type: ignore
+        return await child.ainvoke(x, config)  # type: ignore[arg-type]
 
     bond = uuid.UUID(int=7)
     events = await _collect_events(
@@ -2326,7 +2329,7 @@ async def test_bad_parent_ids() -> None:
 async def test_runnable_generator() -> None:
     """Test async events from sync lambda."""
 
-    async def generator(inputs: AsyncIterator[str]) -> AsyncIterator[str]:
+    async def generator(_: AsyncIterator[str]) -> AsyncIterator[str]:
         yield "1"
         yield "2"
 
@@ -2398,9 +2401,7 @@ async def test_with_explicit_config() -> None:
 
         return await chain.ainvoke(query)
 
-    events = await _collect_events(
-        say_hello.astream_events("meow", version="v2")  # type: ignore
-    )
+    events = await _collect_events(say_hello.astream_events("meow", version="v2"))
 
     assert [
         event["data"]["chunk"].content
@@ -2414,14 +2415,14 @@ async def test_break_astream_events() -> None:
         def __init__(self) -> None:
             self.reset()
 
-        async def __call__(self, input: Any) -> Any:
+        async def __call__(self, value: Any) -> Any:
             self.started = True
             try:
                 await asyncio.sleep(0.5)
             except asyncio.CancelledError:
                 self.cancelled = True
                 raise
-            return input
+            return value
 
         def reset(self) -> None:
             self.started = False
@@ -2434,11 +2435,11 @@ async def test_break_astream_events() -> None:
     outer_cancelled = False
 
     @chain
-    async def sequence(input: Any) -> Any:
+    async def sequence(value: Any) -> Any:
         try:
-            yield await alittlewhile(input)
-            yield await awhile(input)
-            yield await anotherwhile(input)
+            yield await alittlewhile(value)
+            yield await awhile(value)
+            yield await anotherwhile(value)
         except asyncio.CancelledError:
             nonlocal outer_cancelled
             outer_cancelled = True
@@ -2479,14 +2480,14 @@ async def test_cancel_astream_events() -> None:
         def __init__(self) -> None:
             self.reset()
 
-        async def __call__(self, input: Any) -> Any:
+        async def __call__(self, value: Any) -> Any:
             self.started = True
             try:
                 await asyncio.sleep(0.5)
             except asyncio.CancelledError:
                 self.cancelled = True
                 raise
-            return input
+            return value
 
         def reset(self) -> None:
             self.started = False
@@ -2499,11 +2500,11 @@ async def test_cancel_astream_events() -> None:
     outer_cancelled = False
 
     @chain
-    async def sequence(input: Any) -> Any:
+    async def sequence(value: Any) -> Any:
         try:
-            yield await alittlewhile(input)
-            yield await awhile(input)
-            yield await anotherwhile(input)
+            yield await alittlewhile(value)
+            yield await awhile(value)
+            yield await anotherwhile(value)
         except asyncio.CancelledError:
             nonlocal outer_cancelled
             outer_cancelled = True
@@ -2548,7 +2549,6 @@ async def test_cancel_astream_events() -> None:
 
 async def test_custom_event() -> None:
     """Test adhoc event."""
-    from langchain_core.callbacks.manager import adispatch_custom_event
 
     # Ignoring type due to RunnableLamdba being dynamic when it comes to being
     # applied as a decorator to async functions.
@@ -2625,7 +2625,6 @@ async def test_custom_event() -> None:
 
 async def test_custom_event_nested() -> None:
     """Test adhoc event in a nested chain."""
-    from langchain_core.callbacks.manager import adispatch_custom_event
 
     # Ignoring type due to RunnableLamdba being dynamic when it comes to being
     # applied as a decorator to async functions.
@@ -2736,7 +2735,6 @@ async def test_custom_event_root_dispatch() -> None:
     # This just tests that nothing breaks on the path.
     # It shouldn't do anything at the moment, since the tracer isn't configured
     # to handle adhoc events.
-    from langchain_core.callbacks.manager import adispatch_custom_event
 
     # Expected behavior is that the event cannot be dispatched
     with pytest.raises(RuntimeError):
@@ -2750,8 +2748,6 @@ IS_GTE_3_11 = sys.version_info >= (3, 11)
 @pytest.mark.skipif(not IS_GTE_3_11, reason="Requires Python >=3.11")
 async def test_custom_event_root_dispatch_with_in_tool() -> None:
     """Test adhoc event in a nested chain."""
-    from langchain_core.callbacks.manager import adispatch_custom_event
-    from langchain_core.tools import tool
 
     @tool
     async def foo(x: int) -> int:
@@ -2760,9 +2756,7 @@ async def test_custom_event_root_dispatch_with_in_tool() -> None:
         return x + 1
 
     # Ignoring type due to @tool not returning correct type annotations
-    events = await _collect_events(
-        foo.astream_events({"x": 2}, version="v2")  # type: ignore[attr-defined]
-    )
+    events = await _collect_events(foo.astream_events({"x": 2}, version="v2"))
     _assert_events_equal_allow_superset_metadata(
         events,
         [
@@ -2799,6 +2793,5 @@ async def test_custom_event_root_dispatch_with_in_tool() -> None:
 
 def test_default_is_v2() -> None:
     """Test that we default to version="v2"."""
-
     signature = inspect.signature(Runnable.astream_events)
     assert signature.parameters["version"].default == "v2"
