@@ -1,4 +1,3 @@
-import unittest
 import uuid
 from typing import Optional, Union
 
@@ -21,7 +20,9 @@ from langchain_core.messages import (
     SystemMessage,
     ToolMessage,
     convert_to_messages,
+    convert_to_openai_image_block,
     get_buffer_string,
+    is_data_content_block,
     merge_content,
     message_chunk_to_message,
     message_to_dict,
@@ -176,6 +177,22 @@ def test_message_chunks() -> None:
     assert AIMessageChunk(content="") + left == left
     assert right + AIMessageChunk(content="") == right
 
+    # Test ID order of precedence
+    null_id = AIMessageChunk(content="", id=None)
+    default_id = AIMessageChunk(
+        content="", id="run-abc123"
+    )  # LangChain-assigned run ID
+    meaningful_id = AIMessageChunk(content="", id="msg_def456")  # provider-assigned ID
+
+    assert (null_id + default_id).id == "run-abc123"
+    assert (default_id + null_id).id == "run-abc123"
+
+    assert (null_id + meaningful_id).id == "msg_def456"
+    assert (meaningful_id + null_id).id == "msg_def456"
+
+    assert (default_id + meaningful_id).id == "msg_def456"
+    assert (meaningful_id + default_id).id == "msg_def456"
+
 
 def test_chat_message_chunks() -> None:
     assert ChatMessageChunk(role="User", content="I am", id="ai4") + ChatMessageChunk(
@@ -184,7 +201,9 @@ def test_chat_message_chunks() -> None:
         "ChatMessageChunk + ChatMessageChunk should be a ChatMessageChunk"
     )
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError, match="Cannot concatenate ChatMessageChunks with different roles"
+    ):
         ChatMessageChunk(role="User", content="I am") + ChatMessageChunk(
             role="Assistant", content=" indeed."
         )
@@ -290,7 +309,10 @@ def test_function_message_chunks() -> None:
         id="ai5", name="hello", content="I am indeed."
     ), "FunctionMessageChunk + FunctionMessageChunk should be a FunctionMessageChunk"
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match="Cannot concatenate FunctionMessageChunks with different names",
+    ):
         FunctionMessageChunk(name="hello", content="I am") + FunctionMessageChunk(
             name="bye", content=" indeed."
         )
@@ -303,74 +325,56 @@ def test_ai_message_chunks() -> None:
         "AIMessageChunk + AIMessageChunk should be a AIMessageChunk"
     )
 
-    with pytest.raises(ValueError):
+    with pytest.raises(
+        ValueError,
+        match="Cannot concatenate AIMessageChunks with different example values",
+    ):
         AIMessageChunk(example=True, content="I am") + AIMessageChunk(
             example=False, content=" indeed."
         )
 
 
-class TestGetBufferString(unittest.TestCase):
-    def setUp(self) -> None:
-        self.human_msg = HumanMessage(content="human")
-        self.ai_msg = AIMessage(content="ai")
-        self.sys_msg = SystemMessage(content="system")
-        self.func_msg = FunctionMessage(name="func", content="function")
-        self.tool_msg = ToolMessage(tool_call_id="tool_id", content="tool")
-        self.chat_msg = ChatMessage(role="Chat", content="chat")
-        self.tool_calls_msg = AIMessage(content="tool")
+class TestGetBufferString:
+    _HUMAN_MSG = HumanMessage(content="human")
+    _AI_MSG = AIMessage(content="ai")
 
     def test_empty_input(self) -> None:
-        self.assertEqual(get_buffer_string([]), "")
+        assert not get_buffer_string([])
 
     def test_valid_single_message(self) -> None:
-        expected_output = f"Human: {self.human_msg.content}"
-        self.assertEqual(
-            get_buffer_string([self.human_msg]),
-            expected_output,
-        )
+        expected_output = "Human: human"
+        assert get_buffer_string([self._HUMAN_MSG]) == expected_output
 
     def test_custom_human_prefix(self) -> None:
-        prefix = "H"
-        expected_output = f"{prefix}: {self.human_msg.content}"
-        self.assertEqual(
-            get_buffer_string([self.human_msg], human_prefix="H"),
-            expected_output,
-        )
+        expected_output = "H: human"
+        assert get_buffer_string([self._HUMAN_MSG], human_prefix="H") == expected_output
 
     def test_custom_ai_prefix(self) -> None:
-        prefix = "A"
-        expected_output = f"{prefix}: {self.ai_msg.content}"
-        self.assertEqual(
-            get_buffer_string([self.ai_msg], ai_prefix="A"),
-            expected_output,
-        )
+        expected_output = "A: ai"
+        assert get_buffer_string([self._AI_MSG], ai_prefix="A") == expected_output
 
     def test_multiple_msg(self) -> None:
         msgs = [
-            self.human_msg,
-            self.ai_msg,
-            self.sys_msg,
-            self.func_msg,
-            self.tool_msg,
-            self.chat_msg,
-            self.tool_calls_msg,
+            self._HUMAN_MSG,
+            self._AI_MSG,
+            SystemMessage(content="system"),
+            FunctionMessage(name="func", content="function"),
+            ToolMessage(tool_call_id="tool_id", content="tool"),
+            ChatMessage(role="Chat", content="chat"),
+            AIMessage(content="tool"),
         ]
-        expected_output = "\n".join(  # noqa: FLY002
-            [
-                "Human: human",
-                "AI: ai",
-                "System: system",
-                "Function: function",
-                "Tool: tool",
-                "Chat: chat",
-                "AI: tool",
-            ]
-        )
-        self.assertEqual(
-            get_buffer_string(msgs),
-            expected_output,
+        expected_output = (
+            "Human: human\n"
+            "AI: ai\n"
+            "System: system\n"
+            "Function: function\n"
+            "Tool: tool\n"
+            "Chat: chat\n"
+            "AI: tool"
         )
 
+        assert get_buffer_string(msgs) == expected_output
+
 
 def test_multiple_msg() -> None:
     human_msg = HumanMessage(content="human", additional_kwargs={"key": "value"})
@@ -439,11 +443,11 @@ def test_message_chunk_to_message() -> None:
     expected = AIMessage(
         content="I am",
         tool_calls=[
-            create_tool_call(name="tool1", args={"a": 1}, id="1"),  # type: ignore[arg-type]
-            create_tool_call(name="tool2", args={}, id="2"),  # type: ignore[arg-type]
+            create_tool_call(name="tool1", args={"a": 1}, id="1"),
+            create_tool_call(name="tool2", args={}, id="2"),
+            create_tool_call(name="tool3", args={}, id="3"),
         ],
         invalid_tool_calls=[
-            create_invalid_tool_call(name="tool3", args=None, id="3", error=None),
             create_invalid_tool_call(name="tool4", args="abc", id="4", error=None),
         ],
     )
@@ -736,6 +740,7 @@ def test_convert_to_messages() -> None:
                 "tool_call_id": "tool_id2",
                 "content": "Bye!",
                 "artifact": {"foo": 123},
+                "status": "success",
             },
             {"role": "remove", "id": "message_to_remove", "content": ""},
             {
@@ -769,7 +774,12 @@ def test_convert_to_messages() -> None:
             ],
         ),
         ToolMessage(tool_call_id="tool_id", content="Hi!"),
-        ToolMessage(tool_call_id="tool_id2", content="Bye!", artifact={"foo": 123}),
+        ToolMessage(
+            tool_call_id="tool_id2",
+            content="Bye!",
+            artifact={"foo": 123},
+            status="success",
+        ),
         RemoveMessage(id="message_to_remove"),
         HumanMessage(
             content="Now the turn for Larry to ask a question about the book!",
@@ -926,7 +936,7 @@ def test_tool_message_serdes() -> None:
 
 
 class BadObject:
-    """"""
+    pass
 
 
 def test_tool_message_ser_non_serializable() -> None:
@@ -991,7 +1001,7 @@ def test_tool_message_str() -> None:
 
 
 @pytest.mark.parametrize(
-    ["first", "others", "expected"],
+    ("first", "others", "expected"),
     [
         ("", [""], ""),
         ("", [[]], [""]),
@@ -1005,6 +1015,11 @@ def test_tool_message_str() -> None:
         ("foo", [["bar"]], ["foo", "bar"]),
         (["foo"], ["bar"], ["foobar"]),
         (["foo"], [["bar"]], ["foo", "bar"]),
+        (
+            [{"text": "foo"}],
+            [[{"index": 0, "text": "bar"}]],
+            [{"text": "foo"}, {"index": 0, "text": "bar"}],
+        ),
     ],
 )
 def test_merge_content(
@@ -1043,7 +1058,7 @@ def test_message_text() -> None:
     # content dict types: [text], [not text], [no type]
 
     assert HumanMessage(content="foo").text() == "foo"
-    assert AIMessage(content=[]).text() == ""
+    assert not AIMessage(content=[]).text()
     assert AIMessage(content=["foo", "bar"]).text() == "foobar"
     assert (
         AIMessage(
@@ -1083,11 +1098,100 @@ def test_message_text() -> None:
     assert (
         AIMessage(content=[{"text": "hi there"}, "hi"]).text() == "hi"
     )  # missing type: text
-    assert AIMessage(content=[{"type": "nottext", "text": "hi"}]).text() == ""
-    assert AIMessage(content=[]).text() == ""
-    assert (
-        AIMessage(
-            content="", tool_calls=[create_tool_call(name="a", args={"b": 1}, id=None)]
-        ).text()
-        == ""
+    assert not AIMessage(content=[{"type": "nottext", "text": "hi"}]).text()
+    assert not AIMessage(content=[]).text()
+    assert not AIMessage(
+        content="", tool_calls=[create_tool_call(name="a", args={"b": 1}, id=None)]
+    ).text()
+
+
+def test_is_data_content_block() -> None:
+    assert is_data_content_block(
+        {
+            "type": "image",
+            "source_type": "url",
+            "url": "https://...",
+        }
     )
+    assert is_data_content_block(
+        {
+            "type": "image",
+            "source_type": "base64",
+            "data": "<base64 data>",
+            "mime_type": "image/jpeg",
+        }
+    )
+    assert is_data_content_block(
+        {
+            "type": "image",
+            "source_type": "base64",
+            "data": "<base64 data>",
+            "mime_type": "image/jpeg",
+            "cache_control": {"type": "ephemeral"},
+        }
+    )
+    assert is_data_content_block(
+        {
+            "type": "image",
+            "source_type": "base64",
+            "data": "<base64 data>",
+            "mime_type": "image/jpeg",
+            "metadata": {"cache_control": {"type": "ephemeral"}},
+        }
+    )
+
+    assert not is_data_content_block(
+        {
+            "type": "text",
+            "text": "foo",
+        }
+    )
+    assert not is_data_content_block(
+        {
+            "type": "image_url",
+            "image_url": {"url": "https://..."},
+        }
+    )
+    assert not is_data_content_block(
+        {
+            "type": "image",
+            "source_type": "base64",
+        }
+    )
+    assert not is_data_content_block(
+        {
+            "type": "image",
+            "source": "<base64 data>",
+        }
+    )
+
+
+def test_convert_to_openai_image_block() -> None:
+    input_block = {
+        "type": "image",
+        "source_type": "url",
+        "url": "https://...",
+        "cache_control": {"type": "ephemeral"},
+    }
+    expected = {
+        "type": "image_url",
+        "image_url": {"url": "https://..."},
+    }
+    result = convert_to_openai_image_block(input_block)
+    assert result == expected
+
+    input_block = {
+        "type": "image",
+        "source_type": "base64",
+        "data": "<base64 data>",
+        "mime_type": "image/jpeg",
+        "cache_control": {"type": "ephemeral"},
+    }
+    expected = {
+        "type": "image_url",
+        "image_url": {
+            "url": "data:image/jpeg;base64,<base64 data>",
+        },
+    }
+    result = convert_to_openai_image_block(input_block)
+    assert result == expected
