@@ -1,4 +1,3 @@
-# mypy: disable-error-code="annotation-unchecked"
 import sys
 import typing
 from collections.abc import Iterable, Mapping, MutableMapping, Sequence
@@ -15,15 +14,18 @@ from typing import TypedDict as TypingTypedDict
 import pytest
 from pydantic import BaseModel as BaseModelV2Maybe  # pydantic: ignore
 from pydantic import Field as FieldV2Maybe  # pydantic: ignore
-from typing_extensions import (
-    TypedDict as ExtensionsTypedDict,
-)
+from typing_extensions import TypeAlias
+from typing_extensions import TypedDict as ExtensionsTypedDict
 
 try:
-    from typing import Annotated as TypingAnnotated  # type: ignore[attr-defined]
+    from typing import Annotated as TypingAnnotated
 except ImportError:
     TypingAnnotated = ExtensionsAnnotated
 
+
+from importlib.metadata import version
+
+from packaging.version import parse
 from pydantic import BaseModel, Field
 
 from langchain_core.messages import AIMessage, HumanMessage, ToolMessage
@@ -37,7 +39,7 @@ from langchain_core.utils.function_calling import (
 )
 
 
-@pytest.fixture()
+@pytest.fixture
 def pydantic() -> type[BaseModel]:
     class dummy_function(BaseModel):  # noqa: N801
         """Dummy function."""
@@ -48,7 +50,7 @@ def pydantic() -> type[BaseModel]:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def annotated_function() -> Callable:
     def dummy_function(
         arg1: ExtensionsAnnotated[int, "foo"],
@@ -59,7 +61,7 @@ def annotated_function() -> Callable:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def function() -> Callable:
     def dummy_function(arg1: int, arg2: Literal["bar", "baz"]) -> None:
         """Dummy function.
@@ -72,7 +74,7 @@ def function() -> Callable:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def function_docstring_annotations() -> Callable:
     def dummy_function(arg1: int, arg2: Literal["bar", "baz"]) -> None:
         """Dummy function.
@@ -85,7 +87,7 @@ def function_docstring_annotations() -> Callable:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def runnable() -> Runnable:
     class Args(ExtensionsTypedDict):
         arg1: ExtensionsAnnotated[int, "foo"]
@@ -97,7 +99,7 @@ def runnable() -> Runnable:
     return RunnableLambda(dummy_function)
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_tool() -> BaseTool:
     class Schema(BaseModel):
         arg1: int = Field(..., description="foo")
@@ -114,21 +116,21 @@ def dummy_tool() -> BaseTool:
     return DummyFunction()
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_structured_tool() -> StructuredTool:
     class Schema(BaseModel):
         arg1: int = Field(..., description="foo")
         arg2: Literal["bar", "baz"] = Field(..., description="one of 'bar', 'baz'")
 
     return StructuredTool.from_function(
-        lambda x: None,
+        lambda _: None,
         name="dummy_function",
         description="Dummy function.",
         args_schema=Schema,
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_structured_tool_args_schema_dict() -> StructuredTool:
     args_schema = {
         "type": "object",
@@ -143,14 +145,14 @@ def dummy_structured_tool_args_schema_dict() -> StructuredTool:
         "required": ["arg1", "arg2"],
     }
     return StructuredTool.from_function(
-        lambda x: None,
+        lambda _: None,
         name="dummy_function",
         description="Dummy function.",
         args_schema=args_schema,
     )
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_pydantic() -> type[BaseModel]:
     class dummy_function(BaseModel):  # noqa: N801
         """Dummy function."""
@@ -161,7 +163,7 @@ def dummy_pydantic() -> type[BaseModel]:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_pydantic_v2() -> type[BaseModelV2Maybe]:
     class dummy_function(BaseModelV2Maybe):  # noqa: N801
         """Dummy function."""
@@ -174,7 +176,7 @@ def dummy_pydantic_v2() -> type[BaseModelV2Maybe]:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_typing_typed_dict() -> type:
     class dummy_function(TypingTypedDict):  # noqa: N801
         """Dummy function."""
@@ -185,7 +187,7 @@ def dummy_typing_typed_dict() -> type:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_typing_typed_dict_docstring() -> type:
     class dummy_function(TypingTypedDict):  # noqa: N801
         """Dummy function.
@@ -201,7 +203,7 @@ def dummy_typing_typed_dict_docstring() -> type:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_extensions_typed_dict() -> type:
     class dummy_function(ExtensionsTypedDict):  # noqa: N801
         """Dummy function."""
@@ -212,7 +214,7 @@ def dummy_extensions_typed_dict() -> type:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def dummy_extensions_typed_dict_docstring() -> type:
     class dummy_function(ExtensionsTypedDict):  # noqa: N801
         """Dummy function.
@@ -228,7 +230,7 @@ def dummy_extensions_typed_dict_docstring() -> type:
     return dummy_function
 
 
-@pytest.fixture()
+@pytest.fixture
 def json_schema() -> dict:
     return {
         "title": "dummy_function",
@@ -246,7 +248,7 @@ def json_schema() -> dict:
     }
 
 
-@pytest.fixture()
+@pytest.fixture
 def anthropic_tool() -> dict:
     return {
         "name": "dummy_function",
@@ -266,7 +268,7 @@ def anthropic_tool() -> dict:
     }
 
 
-@pytest.fixture()
+@pytest.fixture
 def bedrock_converse_tool() -> dict:
     return {
         "toolSpec": {
@@ -366,7 +368,7 @@ def test_convert_to_openai_function(
         dummy_extensions_typed_dict,
         dummy_extensions_typed_dict_docstring,
     ):
-        actual = convert_to_openai_function(fn)  # type: ignore
+        actual = convert_to_openai_function(fn)
         assert actual == expected
 
     # Test runnables
@@ -387,8 +389,8 @@ def test_convert_to_openai_function(
     assert actual == runnable_expected
 
     # Test simple Tool
-    def my_function(input_string: str) -> str:
-        pass
+    def my_function(_: str) -> str:
+        return ""
 
     tool = Tool(
         name="dummy_function",
@@ -499,6 +501,61 @@ def test_convert_to_openai_function_nested_strict() -> None:
     assert actual == expected
 
 
+def test_convert_to_openai_function_strict_union_of_objects_arg_type() -> None:
+    class NestedA(BaseModel):
+        foo: str
+
+    class NestedB(BaseModel):
+        bar: int
+
+    class NestedC(BaseModel):
+        baz: bool
+
+    def my_function(my_arg: Union[NestedA, NestedB, NestedC]) -> None:
+        """Dummy function."""
+
+    expected = {
+        "name": "my_function",
+        "description": "Dummy function.",
+        "parameters": {
+            "properties": {
+                "my_arg": {
+                    "anyOf": [
+                        {
+                            "properties": {"foo": {"title": "Foo", "type": "string"}},
+                            "required": ["foo"],
+                            "title": "NestedA",
+                            "type": "object",
+                            "additionalProperties": False,
+                        },
+                        {
+                            "properties": {"bar": {"title": "Bar", "type": "integer"}},
+                            "required": ["bar"],
+                            "title": "NestedB",
+                            "type": "object",
+                            "additionalProperties": False,
+                        },
+                        {
+                            "properties": {"baz": {"title": "Baz", "type": "boolean"}},
+                            "required": ["baz"],
+                            "title": "NestedC",
+                            "type": "object",
+                            "additionalProperties": False,
+                        },
+                    ]
+                }
+            },
+            "required": ["my_arg"],
+            "type": "object",
+            "additionalProperties": False,
+        },
+        "strict": True,
+    }
+
+    actual = convert_to_openai_function(my_function, strict=True)
+    assert actual == expected
+
+
 json_schema_no_description_no_params = {
     "title": "dummy_function",
 }
@@ -738,20 +795,25 @@ def test_tool_outputs() -> None:
     assert not response.tool_calls
 
 
-@pytest.mark.parametrize("use_extension_typed_dict", [True, False])
-@pytest.mark.parametrize("use_extension_annotated", [True, False])
+@pytest.mark.parametrize(
+    "typed_dict",
+    [ExtensionsTypedDict, TypingTypedDict],
+    ids=["typing_extensions.TypedDict", "typing.TypedDict"],
+)
+@pytest.mark.parametrize(
+    "annotated",
+    [ExtensionsAnnotated, TypingAnnotated],
+    ids=["typing_extensions.Annotated", "typing.Annotated"],
+)
 def test__convert_typed_dict_to_openai_function(
-    use_extension_typed_dict: bool, use_extension_annotated: bool
+    typed_dict: TypeAlias, annotated: TypeAlias
 ) -> None:
-    typed_dict = ExtensionsTypedDict if use_extension_typed_dict else TypingTypedDict
-    annotated = TypingAnnotated if use_extension_annotated else TypingAnnotated
-
-    class SubTool(typed_dict):
+    class SubTool(typed_dict):  # type: ignore[misc]
         """Subtool docstring."""
 
-        args: annotated[dict[str, Any], {}, "this does bar"]  # noqa: F722  # type: ignore
+        args: annotated[dict[str, Any], {}, "this does bar"]  # noqa: F722
 
-    class Tool(typed_dict):
+    class Tool(typed_dict):  # type: ignore[misc]
         """Docstring.
 
         Args:
@@ -774,7 +836,7 @@ def test__convert_typed_dict_to_openai_function(
         arg12: annotated[dict[str, SubTool], ...]
         arg13: annotated[Mapping[str, SubTool], ...]
         arg14: annotated[MutableMapping[str, SubTool], ...]
-        arg15: annotated[bool, False, "flag"]  # noqa: F821  # type: ignore
+        arg15: annotated[bool, False, "flag"]  # noqa: F821
 
     expected = {
         "name": "Tool",
@@ -981,7 +1043,7 @@ def test__convert_typed_dict_to_openai_function(
 
 @pytest.mark.parametrize("typed_dict", [ExtensionsTypedDict, TypingTypedDict])
 def test__convert_typed_dict_to_openai_function_fail(typed_dict: type) -> None:
-    class Tool(typed_dict):
+    class Tool(typed_dict):  # type: ignore[misc]
         arg1: typing.MutableSet  # Pydantic 2 supports this, but pydantic v1 does not.
 
     # Error should be raised since we're using v1 code path here
@@ -994,12 +1056,13 @@ def test__convert_typed_dict_to_openai_function_fail(typed_dict: type) -> None:
 )
 def test_convert_union_type_py_39() -> None:
     @tool
-    def magic_function(input: int | float) -> str:
+    def magic_function(value: int | str) -> str:  # type: ignore[syntax,unused-ignore] # noqa: ARG001,FA102
         """Compute a magic function."""
+        return ""
 
     result = convert_to_openai_function(magic_function)
-    assert result["parameters"]["properties"]["input"] == {
-        "anyOf": [{"type": "integer"}, {"type": "number"}]
+    assert result["parameters"]["properties"]["value"] == {
+        "anyOf": [{"type": "integer"}, {"type": "string"}]
     }
 
 
@@ -1061,5 +1124,43 @@ def test_convert_to_json_schema(
         dummy_extensions_typed_dict,
         dummy_extensions_typed_dict_docstring,
     ):
-        actual = convert_to_json_schema(fn)  # type: ignore
+        actual = convert_to_json_schema(fn)
         assert actual == expected
+
+
+def test_convert_to_openai_function_nested_strict_2() -> None:
+    def my_function(arg1: dict, arg2: Union[dict, None]) -> None:
+        """Dummy function."""
+
+    expected: dict = {
+        "name": "my_function",
+        "description": "Dummy function.",
+        "parameters": {
+            "type": "object",
+            "properties": {
+                "arg1": {
+                    "additionalProperties": False,
+                    "type": "object",
+                },
+                "arg2": {
+                    "anyOf": [
+                        {"additionalProperties": False, "type": "object"},
+                        {"type": "null"},
+                    ],
+                },
+            },
+            "required": ["arg1", "arg2"],
+            "additionalProperties": False,
+        },
+        "strict": True,
+    }
+
+    # there will be no extra `"additionalProperties": False` when Pydantic < 2.11
+    if parse(version("pydantic")) < parse("2.11"):
+        del expected["parameters"]["properties"]["arg1"]["additionalProperties"]
+        del expected["parameters"]["properties"]["arg2"]["anyOf"][0][
+            "additionalProperties"
+        ]
+
+    actual = convert_to_openai_function(my_function, strict=True)
+    assert actual == expected
